from __future__ import unicode_literals

import logging

from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _

from acls.api import class_permissions
from common.classes import ModelAttribute
from common.utils import encapsulate
from documents.models import Document, DocumentType
from documents.signals import post_document_type_change
from navigation.api import register_links, register_model_list_columns
from navigation.links import link_spacer
from project_setup.api import register_setup
from project_tools.api import register_tool
from rest_api.classes import APIEndPoint

from .api import get_metadata_string
from .classes import DocumentMetadataHelper
from .links import (
    metadata_add, metadata_edit, metadata_multiple_add, metadata_multiple_edit,
    metadata_multiple_remove, metadata_remove, metadata_view,
    setup_document_type_metadata, setup_document_type_metadata_required,
    setup_metadata_type_create, setup_metadata_type_delete,
    setup_metadata_type_edit, setup_metadata_type_list,
    link_documents_missing_required_metadata
)
from .models import DocumentMetadata, DocumentTypeMetadataType, MetadataType
from .permissions import (
    PERMISSION_METADATA_DOCUMENT_ADD, PERMISSION_METADATA_DOCUMENT_EDIT,
    PERMISSION_METADATA_DOCUMENT_REMOVE, PERMISSION_METADATA_DOCUMENT_VIEW
)
from .tasks import task_add_required_metadata_type, task_remove_metadata_type

logger = logging.getLogger(__name__)


@receiver(post_save, dispatch_uid='post_document_type_metadata_type_add', sender=DocumentTypeMetadataType)
def post_document_type_metadata_type_add(sender, instance, created, **kwargs):
    logger.debug('instance: %s', instance)

    if created and instance.required:
        task_add_required_metadata_type.apply_async(kwargs={'document_type_id': instance.document_type.pk, 'metadata_type_id': instance.metadata_type.pk}, queue='metadata')


@receiver(post_delete, dispatch_uid='post_document_type_metadata_type_delete', sender=DocumentTypeMetadataType)
def post_document_type_metadata_type_delete(sender, instance, **kwargs):
    logger.debug('instance: %s', instance)
    task_remove_metadata_type.apply_async(kwargs={'document_type_id': instance.document_type.pk, 'metadata_type_id': instance.metadata_type.pk}, queue='metadata')


@receiver(post_document_type_change, dispatch_uid='post_post_document_type_change_metadata', sender=Document)
def post_post_document_type_change_metadata(sender, instance, **kwargs):
    logger.debug('received post_document_type_change')
    logger.debug('instance: %s', instance)
    # Delete existing document metadata
    for metadata in instance.metadata.all():
        metadata.delete(enforce_required=False)

    # Add new document type metadata types to document
    for document_type_metadata_type in instance.document_type.metadata.filter(required=True):
        DocumentMetadata.objects.create(document=instance, metadata_type=document_type_metadata_type.metadata_type, value=None)


Document.add_to_class('metadata_value_of', DocumentMetadataHelper.constructor)

register_links(['metadata:metadata_add', 'metadata:metadata_edit', 'metadata:metadata_remove', 'metadata:metadata_view'], [metadata_add, metadata_edit, metadata_remove], menu_name='sidebar')
register_links(Document, [metadata_view], menu_name='form_header')
register_links([Document], [metadata_multiple_add, metadata_multiple_edit, metadata_multiple_remove, link_spacer], menu_name='multi_item_links')
register_links(DocumentType, [setup_document_type_metadata, setup_document_type_metadata_required])
register_links(MetadataType, [setup_metadata_type_edit, setup_metadata_type_delete])
register_links([MetadataType, 'metadata:setup_metadata_type_list', 'metadata:setup_metadata_type_create'], [setup_metadata_type_list, setup_metadata_type_create], menu_name='secondary_menu')

register_setup(setup_metadata_type_list)
register_tool(link_documents_missing_required_metadata)

class_permissions(Document, [
    PERMISSION_METADATA_DOCUMENT_ADD, PERMISSION_METADATA_DOCUMENT_EDIT,
    PERMISSION_METADATA_DOCUMENT_REMOVE, PERMISSION_METADATA_DOCUMENT_VIEW,
])

register_model_list_columns(Document, [
    {
        'name': _('Metadata'), 'attribute': encapsulate(lambda x: get_metadata_string(x))
    },
])

APIEndPoint('metadata')
ModelAttribute(Document, 'metadata__metadata_type__name', label=_('Metadata type name'), type_name='query')
ModelAttribute(Document, 'metadata__value', label=_('Metadata type value'), type_name='query')
ModelAttribute(Document, 'metadata', type_name='related', description=_('Queryset containing a MetadataType instance reference and a value for that metadata type'))
ModelAttribute(Document, 'metadata_value_of', label=_('Value of a metadata'), description=_('Return the value of a specific document metadata'), type_name=['property', 'indexing'])



# -*- coding: utf-8 -*-
from south.v2 import DataMigration
from django.db import transaction
from django.db.utils import ProgrammingError


class Migration(DataMigration):
    def forwards(self, orm):
        "Write your forwards methods here."
        # Note: Don't use "from appname.models import ModelName".
        # Use orm.ModelName to refer to models in this application,
        # and orm['appname.ModelName'] for models in other applications.

        try:
            with transaction.atomic():
                for tag in orm['taggit.tag'].objects.all():
                    color = orm.TagProperties.objects.get(tag=tag).color
                    orm.Tag.objects.create(label=tag.name, color=color)
        except ProgrammingError:
            # Ignore this error because it is expected when migrating a new
            # install, in this situation the django-taggit table never existed
            pass

    def backwards(self, orm):
        "Write your backwards methods here."

    models = {
        u'documents.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'documents'", 'null': 'True', 'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'max_length': '48', 'blank': 'True'})
        },
        u'documents.documenttype': {
            'Meta': {'ordering': "['name']", 'object_name': 'DocumentType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'})
        },
        u'taggit.tag': {
            'Meta': {'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100'})
        },
        u'tags.tag': {
            'Meta': {'object_name': 'Tag'},
            'color': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            'document': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['documents.Document']", 'symmetrical': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '128', 'db_index': 'True'})
        },
        u'tags.tagproperties': {
            'Meta': {'object_name': 'TagProperties'},
            'color': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'properties'", 'to': u"orm['tags.Tag']"})
        }
    }

    complete_apps = ['tags']
    symmetrical = True



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'WorkflowState', fields ['label']
        db.delete_unique(u'document_states_workflowstate', ['label'])

        # Adding unique constraint on 'WorkflowState', fields ['workflow', 'label']
        db.create_unique(u'document_states_workflowstate', ['workflow_id', 'label'])

    def backwards(self, orm):
        # Removing unique constraint on 'WorkflowState', fields ['workflow', 'label']
        db.delete_unique(u'document_states_workflowstate', ['workflow_id', 'label'])

        # Adding unique constraint on 'WorkflowState', fields ['label']
        db.create_unique(u'document_states_workflowstate', ['label'])

    models = {
        u'document_states.documenttypeworkflow': {
            'Meta': {'unique_together': "((u'document_type', u'workflow'),)", 'object_name': 'DocumentTypeWorkflow'},
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'workflow': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['document_states.Workflow']"})
        },
        u'document_states.workflow': {
            'Meta': {'object_name': 'Workflow'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        u'document_states.workflowinstance': {
            'Meta': {'unique_together': "((u'document', u'workflow'),)", 'object_name': 'WorkflowInstance'},
            'document': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['documents.Document']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'workflow': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['document_states.Workflow']"})
        },
        u'document_states.workflowinstancelogentry': {
            'Meta': {'object_name': 'WorkflowInstanceLogEntry'},
            'datetime': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'transition': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['document_states.WorkflowTransition']"}),
            'workflow_instance': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'log_entries'", 'to': u"orm['document_states.WorkflowInstance']"})
        },
        u'document_states.workflowstate': {
            'Meta': {'unique_together': "((u'workflow', u'label'),)", 'object_name': 'WorkflowState'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'initial': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'workflow': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'states'", 'to': u"orm['document_states.Workflow']"})
        },
        u'document_states.workflowtransition': {
            'Meta': {'unique_together': "((u'workflow', u'origin_state', u'destination_state'),)", 'object_name': 'WorkflowTransition'},
            'destination_state': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'destinations'", 'to': u"orm['document_states.WorkflowState']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'origin_state': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'origins'", 'to': u"orm['document_states.WorkflowState']"}),
            'workflow': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'transitions'", 'to': u"orm['document_states.Workflow']"})
        },
        u'documents.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'documents'", 'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'default': "u'Uninitialized document'", 'max_length': '255', 'db_index': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "u'eng'", 'max_length': '8'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "u'1ee5fe5a-7e48-4166-a7d0-3b2acc8fb81b'", 'max_length': '48'})
        },
        u'documents.documenttype': {
            'Meta': {'ordering': "['name']", 'object_name': 'DocumentType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'ocr': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        }
    }

    complete_apps = ['document_states']



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from documents.models import DocumentVersion


@python_2_unicode_compatible
class DocumentVersionOCRError(models.Model):
    document_version = models.ForeignKey(DocumentVersion, verbose_name=_('Document version'))
    datetime_submitted = models.DateTimeField(verbose_name=_('Date time submitted'), auto_now=True, db_index=True)
    result = models.TextField(blank=True, null=True, verbose_name=_('Result'))

    def __str__(self):
        return unicode(self.document_version)

    class Meta:
        ordering = ('datetime_submitted',)
        verbose_name = _('Document Version OCR Error')
        verbose_name_plural = _('Document Version OCR Errors')



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting field 'DocumentQueue.state'
        db.delete_column(u'ocr_documentqueue', 'state')

        # Deleting field 'QueueDocument.delay'
        db.delete_column(u'ocr_queuedocument', 'delay')

        # Deleting field 'QueueDocument.state'
        db.delete_column(u'ocr_queuedocument', 'state')

    def backwards(self, orm):
        # Adding field 'DocumentQueue.state'
        db.add_column(u'ocr_documentqueue', 'state',
                      self.gf('django.db.models.fields.CharField')(default='a', max_length=4),
                      keep_default=False)

        # Adding field 'QueueDocument.delay'
        db.add_column(u'ocr_queuedocument', 'delay',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

        # Adding field 'QueueDocument.state'
        db.add_column(u'ocr_queuedocument', 'state',
                      self.gf('django.db.models.fields.CharField')(default='p', max_length=4),
                      keep_default=False)

    models = {
        u'documents.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['documents.DocumentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'max_length': '48', 'blank': 'True'})
        },
        u'documents.documenttype': {
            'Meta': {'ordering': "['name']", 'object_name': 'DocumentType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'})
        },
        u'ocr.documentqueue': {
            'Meta': {'object_name': 'DocumentQueue'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'})
        },
        u'ocr.queuedocument': {
            'Meta': {'ordering': "('datetime_submitted',)", 'object_name': 'QueueDocument'},
            'datetime_submitted': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'db_index': 'True', 'blank': 'True'}),
            'document': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['documents.Document']"}),
            'document_queue': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'documents'", 'to': u"orm['ocr.DocumentQueue']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'node_name': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'result': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['ocr']



from __future__ import unicode_literals

from rest_framework import serializers

from .models import Index, IndexInstanceNode, IndexTemplateNode


class IndexInstanceNodeSerializer(serializers.ModelSerializer):
    documents = serializers.SerializerMethodField('get_documents_count')

    class Meta:
        fields = ('id', 'parent', 'value', 'level', 'documents', 'children')
        model = IndexInstanceNode

    def get_documents_count(self, obj):
        return obj.documents.count()


IndexInstanceNodeSerializer.base_fields['children'] = IndexInstanceNodeSerializer(many=True)


class IndexTemplateNodeSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('id', 'parent', 'index', 'expression', 'enabled', 'link_documents', 'level')
        model = IndexTemplateNode


class IndexSerializer(serializers.ModelSerializer):
    node_templates = IndexTemplateNodeSerializer(read_only=True, many=True)
    instance_root = IndexInstanceNodeSerializer(read_only=True)

    class Meta:
        fields = ('id', 'name', 'title', 'enabled', 'document_types', 'node_templates', 'instance_root')
        model = Index



from __future__ import unicode_literals

from dateutil.parser import parse

from django.contrib.auth.models import User
from django.utils.timezone import now

from smart_settings.api import register_settings

default_available_functions = {
    'current_date': now().date,
}

default_available_models = {
    'User': User
}

default_available_validators = {
    'Parse date and time': lambda input: parse(input).isoformat(),
    'Parse date': lambda input: parse(input).date().isoformat(),
    'Parse time': lambda input: parse(input).time().isoformat()
}

register_settings(
    namespace='metadata',
    module='metadata.settings',
    settings=[
        # Definition
        {'name': 'AVAILABLE_FUNCTIONS', 'global_name': 'METADATA_AVAILABLE_FUNCTIONS', 'default': default_available_functions},
        {'name': 'AVAILABLE_MODELS', 'global_name': 'METADATA_AVAILABLE_MODELS', 'default': default_available_models},
        {'name': 'AVAILABLE_VALIDATORS', 'global_name': 'METADATA_AVAILABLE_VALIDATORS', 'default': default_available_validators},
    ]
)



from __future__ import unicode_literals

__title__ = 'Mayan EDMS'
__version__ = '1.1.1'
__build__ = 0x010101
__author__ = 'Roberto Rosario'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2011-2015 Roberto Rosario'



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'DocumentCheckout.checkout_datetime'
        db.alter_column(u'checkouts_documentcheckout', 'checkout_datetime', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, default=datetime.datetime(2014, 10, 18, 0, 0)))

    def backwards(self, orm):

        # Changing field 'DocumentCheckout.checkout_datetime'
        db.alter_column(u'checkouts_documentcheckout', 'checkout_datetime', self.gf('django.db.models.fields.DateTimeField')(null=True))

    models = {
        u'checkouts.documentcheckout': {
            'Meta': {'object_name': 'DocumentCheckout'},
            'block_new_version': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'checkout_datetime': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'document': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['documents.Document']", 'unique': 'True'}),
            'expiration_datetime': ('django.db.models.fields.DateTimeField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user_content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'user_object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'documents.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'documents'", 'null': 'True', 'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'max_length': '48', 'blank': 'True'})
        },
        u'documents.documenttype': {
            'Meta': {'ordering': "['name']", 'object_name': 'DocumentType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'})
        }
    }

    complete_apps = ['checkouts']



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from .permissions import PERMISSION_MAILING_LINK, PERMISSION_MAILING_SEND_DOCUMENT

send_document_link = {'text': _('Email link'), 'view': 'mailer:send_document_link', 'args': 'object.pk', 'famfam': 'email_link', 'permissions': [PERMISSION_MAILING_LINK]}
send_document = {'text': _('Email document'), 'view': 'mailer:send_document', 'args': 'object.pk', 'famfam': 'email_open', 'permissions': [PERMISSION_MAILING_SEND_DOCUMENT]}



from __future__ import unicode_literals


class NewDocumentVersionNotAllowed(Exception):
    """
    Uploading new versions for this document is not allowed
    Current reasons:  Document is in checked out state
    """
    pass



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from .models import MetadataType


class DocumentMetadataHelper(object):
    @staticmethod
    @property
    def constructor(source_object):
        return DocumentMetadataHelper(source_object)

    def __init__(self, instance):
        self.instance = instance

    def __getattr__(self, name):
        try:
            return self.instance.metadata.get(metadata_type__name=name).value
        except MetadataType.DoesNotExist:
            raise AttributeError(_('\'metadata\' object has no attribute \'%s\'') % name)



from __future__ import absolute_import, unicode_literals

from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _

from documents.models import Document
from main.api import register_maintenance_links
from metadata.models import DocumentMetadata
from navigation.api import register_links, register_top_menu
from project_setup.api import register_setup
from rest_api.classes import APIEndPoint

from .links import (
    document_index_list, document_index_main_menu_link, index_parent,
    index_setup, index_setup_create, index_setup_document_types,
    index_setup_delete, index_setup_edit, index_setup_list, index_setup_view,
    rebuild_index_instances, template_node_create, template_node_delete,
    template_node_edit
)
from .models import Index, IndexTemplateNode, IndexInstanceNode
from .tasks import task_delete_empty_index_nodes, task_index_document


@receiver(post_delete, dispatch_uid='document_index_delete', sender=Document)
def document_index_delete(sender, **kwargs):
    task_delete_empty_index_nodes.apply_async(queue='indexing')


@receiver(post_save, dispatch_uid='document_metadata_index_update', sender=DocumentMetadata)
def document_metadata_index_update(sender, **kwargs):
    task_index_document.apply_async(kwargs=dict(document_id=kwargs['instance'].document.pk), queue='indexing')


@receiver(post_delete, dispatch_uid='document_metadata_index_post_delete', sender=DocumentMetadata)
def document_metadata_index_post_delete(sender, **kwargs):
    task_index_document.apply_async(kwargs=dict(document_id=kwargs['instance'].document.pk), queue='indexing')


register_maintenance_links([rebuild_index_instances], namespace='document_indexing', title=_('Indexes'))

register_links(Document, [document_index_list], menu_name='form_header')
register_links([Index, 'indexing:index_setup_list', 'indexing:index_setup_create'], [index_setup_list, index_setup_create], menu_name='secondary_menu')
register_links(Index, [index_setup_edit, index_setup_view, index_setup_document_types, index_setup_delete])
register_links(IndexInstanceNode, [index_parent])
register_links(IndexTemplateNode, [template_node_create, template_node_edit, template_node_delete])

register_setup(index_setup)

register_top_menu('indexes', document_index_main_menu_link)

APIEndPoint('indexes', app_name='document_indexing')



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _, ugettext

from solo.models import SingletonModel

from permissions.models import StoredPermission

from .api import get_classes
from .classes import AccessObjectClass
from .managers import AccessEntryManager, DefaultAccessEntryManager

logger = logging.getLogger(__name__)


class AccessEntry(models.Model):
    """
    Model that hold the permission, object, actor relationship
    """
    permission = models.ForeignKey(StoredPermission, verbose_name=_('Permission'))

    holder_type = models.ForeignKey(
        ContentType,
        related_name='access_holder',
        limit_choices_to={'model__in': ('user', 'group', 'role')}
    )
    holder_id = models.PositiveIntegerField()
    holder_object = generic.GenericForeignKey(
        ct_field='holder_type',
        fk_field='holder_id'
    )

    content_type = models.ForeignKey(
        ContentType,
        related_name='object_content_type'
    )
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey(
        ct_field='content_type',
        fk_field='object_id'
    )

    objects = AccessEntryManager()

    class Meta:
        verbose_name = _('Access entry')
        verbose_name_plural = _('Access entries')

    def __unicode__(self):
        return '%s: %s' % (self.content_type, self.content_object)


class DefaultAccessEntry(models.Model):
    """
    Model that holds the permission, class, actor relationship, that will
    be added upon the creation of an instance of said class
    """
    @classmethod
    def get_classes(cls):
        return [AccessObjectClass.encapsulate(cls) for cls in get_classes()]

    permission = models.ForeignKey(StoredPermission, verbose_name=_('Permission'))

    holder_type = models.ForeignKey(
        ContentType,
        limit_choices_to={'model__in': ('user', 'group', 'role')},
        related_name='default_access_entry_holder'
    )
    holder_id = models.PositiveIntegerField()
    holder_object = generic.GenericForeignKey(
        ct_field='holder_type',
        fk_field='holder_id'
    )

    content_type = models.ForeignKey(
        ContentType,
        related_name='default_access_entry_class'
    )

    objects = DefaultAccessEntryManager()

    class Meta:
        verbose_name = _('Default access entry')
        verbose_name_plural = _('Default access entries')

    def __unicode__(self):
        return '%s: %s' % (self.content_type, self.content_object)


class CreatorSingletonManager(models.Manager):
    def passthru_check(self, holder, creator=None):
        if isinstance(holder, self.model):
            # TODO: raise explicit error if is instance and creator=None
            return creator
        else:
            return holder


class CreatorSingleton(SingletonModel):
    objects = CreatorSingletonManager()

    def __unicode__(self):
        return ugettext('Creator')

    class Meta:
        verbose_name = _('Creator')
        verbose_name_plural = _('Creator')



from __future__ import unicode_literals

from acls.api import class_permissions
from acls.permissions import ACLS_EDIT_ACL, ACLS_VIEW_ACL
from documents.models import Document
from navigation.api import register_links
from project_setup.api import register_setup

from .classes import ResolvedSmartLink
from .links import (
    smart_link_acl_list, smart_link_create, smart_link_condition_create,
    smart_link_condition_delete, smart_link_condition_edit,
    smart_link_condition_list, smart_link_delete, smart_link_document_types,
    smart_link_edit, smart_link_instance_view,
    smart_link_instances_for_document, smart_link_list, smart_link_setup
)
from .models import SmartLink, SmartLinkCondition
from .permissions import (
    PERMISSION_SMART_LINK_DELETE, PERMISSION_SMART_LINK_EDIT,
    PERMISSION_SMART_LINK_VIEW
)

register_links(Document, [smart_link_instances_for_document], menu_name='form_header')
register_links(SmartLink, [smart_link_edit, smart_link_document_types, smart_link_condition_list, smart_link_acl_list, smart_link_delete])
register_links([SmartLink, 'linking:smart_link_list', 'linking:smart_link_create'], [smart_link_list, smart_link_create], menu_name='secondary_menu')
register_links(SmartLinkCondition, [smart_link_condition_edit, smart_link_condition_delete])
register_links(ResolvedSmartLink, [smart_link_instance_view])
register_links(['linking:smart_link_condition_list', 'linking:smart_link_condition_create', 'linking:smart_link_condition_edit', 'linking:smart_link_condition_delete'], [smart_link_condition_create], menu_name='sidebar')

register_setup(smart_link_setup)

class_permissions(SmartLink, [
    ACLS_EDIT_ACL, ACLS_VIEW_ACL, PERMISSION_SMART_LINK_DELETE,
    PERMISSION_SMART_LINK_EDIT, PERMISSION_SMART_LINK_VIEW
])



from __future__ import unicode_literals

from django.dispatch import receiver

from south.signals import post_migrate

from navigation.api import register_links
from project_setup.api import register_setup

from .classes import (
    AccessHolder, AccessObject, AccessObjectClass, ClassAccessHolder
)
from .links import (
    acl_class_acl_detail, acl_class_acl_list, acl_class_grant, acl_class_list,
    acl_class_new_holder_for, acl_class_revoke, acl_detail, acl_grant,
    acl_holder_new, acl_revoke, acl_setup_valid_classes
)
from .models import CreatorSingleton

register_links([AccessObject], [acl_holder_new], menu_name='sidebar')
register_links(AccessObjectClass, [acl_class_acl_list, acl_class_new_holder_for])
register_links(AccessHolder, [acl_detail])
register_links(['acls:acl_setup_valid_classes', 'acls:acl_class_acl_list', 'acls:acl_class_new_holder_for', 'acls:acl_class_acl_detail', 'acls:acl_class_multiple_grant', 'acls:acl_class_multiple_revoke'], [acl_class_list], menu_name='secondary_menu')
register_links(ClassAccessHolder, [acl_class_acl_detail])
register_links(['acls:acl_detail'], [acl_grant, acl_revoke], menu_name='multi_item_links')
register_links(['acls:acl_class_acl_detail'], [acl_class_grant, acl_class_revoke], menu_name='multi_item_links')
register_setup(acl_setup_valid_classes)


@receiver(post_migrate, dispatch_uid='create_creator_user')
def create_creator_user(sender, **kwargs):
    if kwargs['app'] == 'acls':
        CreatorSingleton.objects.get_or_create()



from __future__ import unicode_literals

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from .api import tool_items


def tools_list(request):
    context = {
        'object_navigation_links': tool_items,
        'title': _('Tools'),
    }

    return render_to_response('main/generic_list_horizontal.html', context,
                              context_instance=RequestContext(request))



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from events.classes import Event

event_document_create = Event(name='documents_document_create', label=_('Document created'))
event_document_properties_edit = Event(name='documents_document_edit', label=_('Document properties edited'))
event_document_type_change = Event(name='documents_document_type_change', label=_('Document type changed'))



from __future__ import unicode_literals

from project_tools.api import register_tool

from .classes import APIEndPoint
from .links import link_api, link_api_documentation

APIEndPoint('rest_api')

register_tool(link_api)
register_tool(link_api_documentation)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'HistoryType'
        db.create_table(u'history_historytype', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('namespace', self.gf('django.db.models.fields.CharField')(max_length=64)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=64)),
        ))
        db.send_create_signal(u'history', ['HistoryType'])

        # Adding unique constraint on 'HistoryType', fields ['namespace', 'name']
        db.create_unique(u'history_historytype', ['namespace', 'name'])

        # Adding model 'History'
        db.create_table(u'history_history', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('datetime', self.gf('django.db.models.fields.DateTimeField')()),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('history_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['history.HistoryType'])),
            ('dictionary', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal(u'history', ['History'])

    def backwards(self, orm):
        # Removing unique constraint on 'HistoryType', fields ['namespace', 'name']
        db.delete_unique(u'history_historytype', ['namespace', 'name'])

        # Deleting model 'HistoryType'
        db.delete_table(u'history_historytype')

        # Deleting model 'History'
        db.delete_table(u'history_history')

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'history.history': {
            'Meta': {'ordering': "('-datetime',)", 'object_name': 'History'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'datetime': ('django.db.models.fields.DateTimeField', [], {}),
            'dictionary': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'history_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['history.HistoryType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'history.historytype': {
            'Meta': {'ordering': "('namespace', 'name')", 'unique_together': "(('namespace', 'name'),)", 'object_name': 'HistoryType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'namespace': ('django.db.models.fields.CharField', [], {'max_length': '64'})
        }
    }

    complete_apps = ['history']



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from events.classes import Event

event_document_check_out = Event(name='checkouts_document_check_out', label=_('Document checked out'))
event_document_check_in = Event(name='checkouts_document_check_in', label=_('Document checked in'))
event_document_auto_check_in = Event(name='checkouts_document_auto_check_in', label=_('Document automatically checked in'))
event_document_forceful_check_in = Event(name='checkouts_document_forceful_check_in', label=_('Document forcefully checked in'))



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions.models import PermissionNamespace, Permission

namespace = PermissionNamespace('document_states', _('States'))
PERMISSION_WORKFLOW_CREATE = Permission.objects.register(namespace, 'workflow_create', _('Create workflows'))
PERMISSION_WORKFLOW_DELETE = Permission.objects.register(namespace, 'workflow_delte', _('Delete workflows'))
PERMISSION_WORKFLOW_EDIT = Permission.objects.register(namespace, 'workflow_edit', _('Edit workflows'))
PERMISSION_WORKFLOW_VIEW = Permission.objects.register(namespace, 'workflow_view', _('View workflows'))

PERMISSION_DOCUMENT_WORKFLOW_VIEW = Permission.objects.register(namespace, 'document_workflow_view', _('View document workflows'))
PERMISSION_DOCUMENT_WORKFLOW_TRANSITION = Permission.objects.register(namespace, 'document_workflow_transition', _('Transition document workflows'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging
import os
import tempfile
import types

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.utils.datastructures import MultiValueDict
from django.utils.http import urlquote as django_urlquote
from django.utils.http import urlencode as django_urlencode
from django.utils.importlib import import_module
from django.utils.translation import ugettext_lazy as _

logger = logging.getLogger(__name__)


def urlquote(link=None, get=None):
    """
    This method does both: urlquote() and urlencode()

    urlqoute(): Quote special characters in 'link'

    urlencode(): Map dictionary to query string key=value&...

    HTML escaping is not done.

    Example:

      urlquote('/wiki/Python_(programming_language)')     --> '/wiki/Python_%28programming_language%29'
      urlquote('/mypath/', {'key': 'value'})              --> '/mypath/?key=value'
      urlquote('/mypath/', {'key': ['value1', 'value2']}) --> '/mypath/?key=value1&key=value2'
      urlquote({'key': ['value1', 'value2']})             --> 'key=value1&key=value2'
    """
    if get is None:
        get = []

    assert link or get
    if isinstance(link, dict):
        # urlqoute({'key': 'value', 'key2': 'value2'}) --> key=value&key2=value2
        assert not get, get
        get = link
        link = ''
    assert isinstance(get, dict), 'wrong type "%s", dict required' % type(get)
    # assert not (link.startswith('http://') or link.startswith('https://')), \
    #    'This method should only quote the url path.
    #    It should not start with http(s)://  (%s)' % (
    #    link)
    if get:
        # http://code.djangoproject.com/ticket/9089
        if isinstance(get, MultiValueDict):
            get = get.lists()
        if link:
            link = '%s?' % django_urlquote(link)
        return '%s%s' % (link, django_urlencode(get, doseq=True))
    else:
        return django_urlquote(link)


def return_attrib(obj, attrib, arguments=None):
    try:
        if isinstance(attrib, types.FunctionType):
            return attrib(obj)
        elif isinstance(obj, types.DictType) or isinstance(obj, types.DictionaryType):
            return obj[attrib]
        else:
            result = reduce(getattr, attrib.split('.'), obj)
            if isinstance(result, types.MethodType):
                if arguments:
                    return result(**arguments)
                else:
                    return result()
            else:
                return result
    except Exception as exception:
        if settings.DEBUG:
            return 'Attribute error: %s; %s' % (attrib, exception)
        else:
            pass


# http://snippets.dzone.com/posts/show/5434
# http://snippets.dzone.com/user/jakob
def pretty_size(size, suffixes=None):
    suffixes = suffixes or [
        ('B', 1024L), ('K', 1048576L), ('M', 1073741824L),
        ('G', 1099511627776L), ('T', 1125899906842624L)
    ]

    for suf, lim in suffixes:
        if size > lim:
            continue
        else:
            try:
                return round(size / float(lim / 1024L), 2).__str__() + suf
            except ZeroDivisionError:
                return 0


def pretty_size_10(size):
    return pretty_size(
        size,
        suffixes=[
            ('B', 1000L), ('K', 1000000L), ('M', 1000000000L),
            ('G', 1000000000000L), ('T', 1000000000000000L)
        ])


def return_type(value):
    if isinstance(value, types.FunctionType):
        return value.__doc__ if value.__doc__ else _('Function found')
    elif isinstance(value, types.ClassType):
        return '%s.%s' % (value.__class__.__module__, value.__class__.__name__)
    elif isinstance(value, types.TypeType):
        return '%s.%s' % (value.__module__, value.__name__)
    elif isinstance(value, types.DictType) or isinstance(value, types.DictionaryType):
        return ', '.join(list(value))
    else:
        return value


# http://stackoverflow.com/questions/4248399/page-range-for-printing-algorithm
def parse_range(astr):
    result = set()
    for part in astr.split(','):
        x = part.split('-')
        result.update(range(int(x[0]), int(x[-1]) + 1))
    return sorted(result)


def generate_choices_w_labels(choices, display_object_type=True):
    results = []
    for choice in choices:
        ct = ContentType.objects.get_for_model(choice)
        label = unicode(choice)
        if isinstance(choice, User):
            label = choice.get_full_name() if choice.get_full_name() else choice

        if display_object_type:
            verbose_name = unicode(getattr(choice._meta, 'verbose_name', ct.name))
            results.append(('%s,%s' % (ct.model, choice.pk), '%s: %s' % (verbose_name, label)))
        else:
            results.append(('%s,%s' % (ct.model, choice.pk), '%s' % (label)))

    # Sort results by the label not the key value
    return sorted(results, key=lambda x: x[1])


def get_object_name(obj, display_object_type=True):
    ct_label = ContentType.objects.get_for_model(obj).name
    if isinstance(obj, User):
        label = obj.get_full_name() if obj.get_full_name() else obj
    else:
        label = unicode(obj)

    if display_object_type:
        try:
            verbose_name = unicode(obj._meta.verbose_name)
        except AttributeError:
            verbose_name = ct_label

        return '%s: %s' % (verbose_name, label)
    else:
        return '%s' % (label)


def validate_path(path):
    if not os.path.exists(path):
        # If doesn't exist try to create it
        try:
            os.mkdir(path)
        except Exception as exception:
            logger.debug('unhandled exception: %s', exception)
            return False

    # Check if it is writable
    try:
        fd, test_filepath = tempfile.mkstemp(dir=path)
        os.close(fd)
        os.unlink(test_filepath)
    except Exception as exception:
        logger.debug('unhandled exception: %s', exception)
        return False

    return True


def encapsulate(function):
    # Workaround Django ticket 15791
    # Changeset 16045
    # http://stackoverflow.com/questions/6861601/cannot-resolve-callable-context-variable/6955045#6955045
    return lambda: function


def get_descriptor(file_input, read=True):
    try:
        # Is it a file like object?
        file_input.seek(0)
    except AttributeError:
        # If not, try open it.
        if read:
            return open(file_input, 'rb')
        else:
            return open(file_input, 'wb')
    else:
        return file_input


# http://stackoverflow.com/questions/123198/how-do-i-copy-a-file-in-python
def copyfile(source, destination, buffer_size=1024 * 1024):
    """
    Copy a file from source to dest. source and dest
    can either be strings or any object with a read or
    write method, like StringIO for example.
    """
    source_descriptor = get_descriptor(source)
    destination_descriptor = get_descriptor(destination, read=False)

    while True:
        copy_buffer = source_descriptor.read(buffer_size)
        if copy_buffer:
            destination_descriptor.write(copy_buffer)
        else:
            break

    source_descriptor.close()
    destination_descriptor.close()


def load_backend(backend_string):
    logger.debug('loading: %s', backend_string)
    module_name, klass = backend_string.rsplit('.', 1)

    try:
        return getattr(import_module(module_name), klass)
    except ImportError as exception:
        logger.debug('error importing: %s; %s', backend_string, exception)
        raise


def fs_cleanup(filename, suppress_exceptions=True):
    """
    Tries to remove the given filename. Ignores non-existent files
    """
    try:
        os.remove(filename)
    except OSError:
        if suppress_exceptions:
            pass
        else:
            raise



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):
    depends_on = (
        ('events', '0002_migrate_history_data'),
    )

    def forwards(self, orm):
        # Removing unique constraint on 'HistoryType', fields ['namespace', 'name']
        db.delete_unique(u'history_historytype', ['namespace', 'name'])

        # Deleting model 'History'
        db.delete_table(u'history_history')

        # Deleting model 'HistoryType'
        db.delete_table(u'history_historytype')

    def backwards(self, orm):
        # Adding model 'History'
        db.create_table(u'history_history', (
            ('history_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['history.HistoryType'])),
            ('dictionary', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('datetime', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal(u'history', ['History'])

        # Adding model 'HistoryType'
        db.create_table(u'history_historytype', (
            ('namespace', self.gf('django.db.models.fields.CharField')(max_length=64)),
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=64)),
        ))
        db.send_create_signal(u'history', ['HistoryType'])

        # Adding unique constraint on 'HistoryType', fields ['namespace', 'name']
        db.create_unique(u'history_historytype', ['namespace', 'name'])

    models = {}

    complete_apps = ['history']



from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _

from documents.models import Document, DocumentType

from .managers import MetadataTypeManager
from .settings import AVAILABLE_VALIDATORS


class MetadataType(models.Model):
    """
    Define a type of metadata
    """
    name = models.CharField(unique=True, max_length=48, verbose_name=_('Name'), help_text=_('Do not use python reserved words, or spaces.'))
    # TODO: normalize 'title' to 'label'
    title = models.CharField(max_length=48, verbose_name=_('Title'))
    default = models.CharField(max_length=128, blank=True, null=True,
                               verbose_name=_('Default'),
                               help_text=_('Enter a string to be evaluated.'))
    # TODO: Add enable_lookup boolean to allow users to switch the lookup on and
    # off without losing the lookup expression
    lookup = models.TextField(blank=True, null=True,
                              verbose_name=_('Lookup'),
                              help_text=_('Enter a string to be evaluated that returns an iterable.'))
    validation = models.CharField(blank=True, choices=zip(AVAILABLE_VALIDATORS, AVAILABLE_VALIDATORS), max_length=64, verbose_name=_('Validation function name'))
    # TODO: Find a different way to let users know what models and functions are
    # available now that we removed these from the help_text
    objects = MetadataTypeManager()

    def __unicode__(self):
        return self.title

    def natural_key(self):
        return (self.name,)

    class Meta:
        ordering = ('title',)
        verbose_name = _('Metadata type')
        verbose_name_plural = _('Metadata types')


class DocumentMetadata(models.Model):
    """
    Link a document to a specific instance of a metadata type with it's
    current value
    """
    document = models.ForeignKey(Document, related_name='metadata', verbose_name=_('Document'))
    metadata_type = models.ForeignKey(MetadataType, verbose_name=_('Type'))
    value = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Value'), db_index=True)

    def __unicode__(self):
        return unicode(self.metadata_type)

    def save(self, *args, **kwargs):
        if self.metadata_type.pk not in self.document.document_type.metadata.values_list('metadata_type', flat=True):
            raise ValidationError(_('Metadata type is not valid for this document type.'))

        return super(DocumentMetadata, self).save(*args, **kwargs)

    def delete(self, enforce_required=True, *args, **kwargs):
        if enforce_required and self.metadata_type.pk in self.document.document_type.metadata.filter(required=True).values_list('metadata_type', flat=True):
            raise ValidationError(_('Metadata type is required for this document type.'))

        return super(DocumentMetadata, self).delete(*args, **kwargs)

    class Meta:
        unique_together = ('document', 'metadata_type')
        verbose_name = _('Document metadata')
        verbose_name_plural = _('Document metadata')


class DocumentTypeMetadataType(models.Model):
    document_type = models.ForeignKey(DocumentType, related_name='metadata', verbose_name=_('Document type'))
    metadata_type = models.ForeignKey(MetadataType, verbose_name=_('Metadata type'))
    required = models.BooleanField(default=False, verbose_name=_('Required'))

    def __unicode__(self):
        return unicode(self.metadata_type)

    class Meta:
        unique_together = ('document_type', 'metadata_type')
        verbose_name = _('Document type metadata type options')
        verbose_name_plural = _('Document type metadata types options')



from __future__ import absolute_import

from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client

import common

TEST_ADMIN_EMAIL = 'admin@admin.com'
TEST_ADMIN_PASSWORD = 'test_admin_password'
TEST_ADMIN_USERNAME = 'test_admin'


class UserLoginTestCase(TestCase):
    """
    Test that users can login via the supported authentication methods
    """

    def setUp(self):
        self.admin_user = User.objects.create_superuser(username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL, password=TEST_ADMIN_PASSWORD)
        self.client = Client()

    def test_normal_behaviour(self):
        setattr(common.views, 'LOGIN_METHOD', 'username')
        response = self.client.get(reverse('document_list'))
        self.assertRedirects(response, str(settings.LOGIN_URL))

    def test_username_login(self):
        setattr(common.views, 'LOGIN_METHOD', 'username')
        logged_in = self.client.login(username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD)
        self.assertTrue(logged_in)
        response = self.client.get(reverse('document_list'))
        # We didn't get redirected to the login URL
        self.assertEqual(response.status_code, 200)

    def test_email_login(self):
        with self.settings(COMMON_LOGIN_METHOD='email', AUTHENTICATION_BACKENDS=('common.auth.email_auth_backend.EmailAuthBackend',)):
            setattr(common.views, 'LOGIN_METHOD', 'email')
            logged_in = self.client.login(username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD)
            self.assertFalse(logged_in)

            logged_in = self.client.login(email=TEST_ADMIN_EMAIL, password=TEST_ADMIN_PASSWORD)
            self.assertTrue(logged_in)

            response = self.client.get(reverse('document_list'))
            # We didn't get redirected to the login URL
            self.assertEqual(response.status_code, 200)

    def test_username_login_via_views(self):
        setattr(common.views, 'LOGIN_METHOD', 'username')
        response = self.client.get(reverse('document_list'))
        self.assertRedirects(response, str(settings.LOGIN_URL))

        response = self.client.post(settings.LOGIN_URL, {'username': TEST_ADMIN_USERNAME, 'password': TEST_ADMIN_PASSWORD}, follow=True)
        response = self.client.get(reverse('document_list'))
        # We didn't get redirected to the login URL
        self.assertEqual(response.status_code, 200)

    def test_email_login_via_views(self):
        with self.settings(COMMON_LOGIN_METHOD='email', AUTHENTICATION_BACKENDS=('common.auth.email_auth_backend.EmailAuthBackend',)):
            setattr(common.views, 'LOGIN_METHOD', 'email')
            response = self.client.get(reverse('document_list'))
            self.assertRedirects(response, str(settings.LOGIN_URL))

            response = self.client.post(settings.LOGIN_URL, {'email': TEST_ADMIN_EMAIL, 'password': TEST_ADMIN_PASSWORD}, follow=True)
            response = self.client.get(reverse('document_list'))
            self.assertEqual(response.status_code, 200)



from __future__ import absolute_import, unicode_literals

from datetime import timedelta

from acls.api import class_permissions
from documents.models import Document
from mayan.celery import app
from navigation.api import register_links, register_top_menu
from rest_api.classes import APIEndPoint

from .links import (
    checkin_document, checkout_document, checkout_info, checkout_list
)
from .models import DocumentCheckout
from .permissions import (
    PERMISSION_DOCUMENT_CHECKIN, PERMISSION_DOCUMENT_CHECKIN_OVERRIDE,
    PERMISSION_DOCUMENT_CHECKOUT, PERMISSION_DOCUMENT_RESTRICTIONS_OVERRIDE
)

CHECK_EXPIRED_CHECK_OUTS_INTERVAL = 60  # Lowest check out expiration allowed


def initialize_document_checkout_extra_methods():
    Document.add_to_class('is_checked_out', lambda document: DocumentCheckout.objects.is_document_checked_out(document))
    Document.add_to_class('check_in', lambda document, user=None: DocumentCheckout.objects.check_in_document(document, user))
    Document.add_to_class('checkout_info', lambda document: DocumentCheckout.objects.document_checkout_info(document))
    Document.add_to_class('checkout_state', lambda document: DocumentCheckout.objects.document_checkout_state(document))
    Document.add_to_class('is_new_versions_allowed', lambda document, user=None: DocumentCheckout.objects.is_document_new_versions_allowed(document, user))


app.conf.CELERYBEAT_SCHEDULE.update({
    'check_expired_check_outs': {
        'task': 'checkouts.tasks.task_check_expired_check_outs',
        'schedule': timedelta(seconds=CHECK_EXPIRED_CHECK_OUTS_INTERVAL),
        'options': {'queue': 'checkouts'}
    },
})

class_permissions(Document, [
    PERMISSION_DOCUMENT_CHECKOUT,
    PERMISSION_DOCUMENT_CHECKIN,
    PERMISSION_DOCUMENT_CHECKIN_OVERRIDE,
    PERMISSION_DOCUMENT_RESTRICTIONS_OVERRIDE
])

initialize_document_checkout_extra_methods()

register_links(Document, [checkout_info], menu_name='form_header')
register_links(['checkouts:checkout_info', 'checkouts:checkout_document', 'checkouts:checkin_document'], [checkout_document, checkin_document], menu_name="sidebar")
register_top_menu(name='checkouts', link=checkout_list)

APIEndPoint('checkouts')



from __future__ import unicode_literals

from smart_settings.api import register_settings

available_indexing_functions = {
}

register_settings(
    namespace='document_indexing',
    module='document_indexing.settings',
    settings=[
        # Definition
        {'name': 'AVAILABLE_INDEXING_FUNCTIONS', 'global_name': 'DOCUMENT_INDEXING_AVAILABLE_INDEXING_FUNCTIONS', 'default': available_indexing_functions},
    ]
)



from __future__ import unicode_literals

from django import forms
from django.core.exceptions import ValidationError
from django.forms.formsets import formset_factory
from django.utils.translation import ugettext_lazy as _

from .models import MetadataType
from .settings import AVAILABLE_FUNCTIONS, AVAILABLE_MODELS, AVAILABLE_VALIDATORS


class MetadataForm(forms.Form):

    def clean_value(self):
        metadata_type = MetadataType.objects.get(pk=self.cleaned_data['id'])

        try:
            validation_function = AVAILABLE_VALIDATORS[metadata_type.validation]
        except KeyError:
            # User entered a validation function name, but was not found
            # Return value entered as is
            return self.cleaned_data['value']
        else:
            try:
                # If it is a parsing function we should get a value
                # If it is a validation function we get nothing on success
                result = validation_function(self.cleaned_data['value'])
            except Exception as exception:
                # If it is a validation function and an exception is raise
                # we wrap that into a new ValidationError exception
                # If the function exception is a ValidationError itself the
                # error messages will be in a 'messages' property, so we
                # contatenate them.
                # Otherwise we extract whatever single message the exception
                # included.
                try:
                    message = ', '.join(exception.messages)
                except AttributeError:
                    message = unicode(exception)

                raise ValidationError(_('Invalid value: %(message)s'), params={'message': message}, code='invalid')
            else:
                # Return the result if it was a parsing function
                # If it was a validation function and passed correctly we return
                # the original input value
                return result or self.cleaned_data['value']

        # If a validation function was never specified we return the original
        # value
        return self.cleaned_data['value']

    def __init__(self, *args, **kwargs):
        super(MetadataForm, self).__init__(*args, **kwargs)

        # Set form fields initial values
        if 'initial' in kwargs:
            self.metadata_type = kwargs['initial'].pop('metadata_type', None)
            required = kwargs['initial'].pop('required', False)
            required_string = ''

            if required:
                self.fields['value'].required = True
                required_string = ' (%s)' % _('Required')
            else:
                self.fields['value'].required = False

            self.fields['name'].initial = '%s%s' % ((self.metadata_type.title if self.metadata_type.title else self.metadata_type.name), required_string)
            self.fields['id'].initial = self.metadata_type.pk

            if self.metadata_type.lookup:
                try:
                    choices = eval(self.metadata_type.lookup, AVAILABLE_MODELS)
                    self.fields['value'] = forms.ChoiceField(label=self.fields['value'].label)
                    choices = zip(choices, choices)
                    if not required:
                        choices.insert(0, ('', '------'))
                    self.fields['value'].choices = choices
                    self.fields['value'].required = required
                except Exception as exception:
                    self.fields['value'].initial = exception
                    self.fields['value'].widget = forms.TextInput(attrs={'readonly': 'readonly'})

            if self.metadata_type.default:
                try:
                    self.fields['value'].initial = eval(self.metadata_type.default, AVAILABLE_FUNCTIONS)
                except Exception as exception:
                    self.fields['value'].initial = exception

    id = forms.CharField(label=_('ID'), widget=forms.HiddenInput)

    name = forms.CharField(label=_('Name'), required=False, widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    value = forms.CharField(label=_('Value'), required=False)
    update = forms.BooleanField(initial=True, label=_('Update'), required=False)

MetadataFormSet = formset_factory(MetadataForm, extra=0)


class AddMetadataForm(forms.Form):
    metadata_type = forms.ModelChoiceField(queryset=MetadataType.objects.all(), label=_('Metadata type'))

    def __init__(self, *args, **kwargs):
        document_type = kwargs.pop('document_type')
        super(AddMetadataForm, self).__init__(*args, **kwargs)
        self.fields['metadata_type'].queryset = document_type.metadata.all()


class MetadataRemoveForm(MetadataForm):
    update = forms.BooleanField(initial=False, label=_('Remove'), required=False)

    def __init__(self, *args, **kwargs):
        super(MetadataRemoveForm, self).__init__(*args, **kwargs)
        self.fields.pop('value')


MetadataRemoveFormSet = formset_factory(MetadataRemoveForm, extra=0)


class MetadataTypeForm(forms.ModelForm):
    class Meta:
        fields = ('name', 'title', 'default', 'lookup', 'validation')
        model = MetadataType



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions.models import Permission, PermissionNamespace

mailer_namespace = PermissionNamespace('mailing', _('Mailing'))

PERMISSION_MAILING_LINK = Permission.objects.register(mailer_namespace, 'mail_link', _('Send document link via email'))
PERMISSION_MAILING_SEND_DOCUMENT = Permission.objects.register(mailer_namespace, 'mail_document', _('Send document via email'))



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

COLOR_RED = 'red'
COLOR_BLUE = 'blu'
COLOR_MAGENTA = 'mag'
COLOR_CYAN = 'cya'
COLOR_YELLOW = 'yel'
COLOR_GREENYELLOW = 'gry'
COLOR_CORAL = 'crl'
COLOR_KHAKI = 'kki'
COLOR_LIGHTGREY = 'lig'
COLOR_ORANGE = 'org'

COLOR_CHOICES = (
    (COLOR_BLUE, _('Blue')),
    (COLOR_CYAN, _('Cyan')),
    (COLOR_CORAL, _('Coral')),
    (COLOR_GREENYELLOW, _('Green-Yellow')),
    (COLOR_KHAKI, _('Khaki')),
    (COLOR_LIGHTGREY, _('LightGrey')),
    (COLOR_MAGENTA, _('Magenta')),
    (COLOR_RED, _('Red')),
    (COLOR_ORANGE, _('Orange')),
    (COLOR_YELLOW, _('Yellow'))
)

COLOR_CODES = (
    (COLOR_RED, 'red'),
    (COLOR_BLUE, 'blue'),
    (COLOR_MAGENTA, 'magenta'),
    (COLOR_CYAN, 'cyan'),
    (COLOR_YELLOW, 'yellow'),
    (COLOR_GREENYELLOW, 'greenyellow '),
    (COLOR_CORAL, 'coral'),
    (COLOR_KHAKI, 'khaki'),
    (COLOR_ORANGE, 'orange'),
    (COLOR_LIGHTGREY, 'lightgrey'),
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


def has_usable_password(context):
    return context['request'].user.has_usable_password


link_password_change = {'text': _('Change password'), 'view': 'common:password_change_view', 'famfam': 'computer_key', 'condition': has_usable_password}
link_current_user_details = {'text': _('User details'), 'view': 'common:current_user_details', 'famfam': 'vcard'}
link_current_user_edit = {'text': _('Edit details'), 'view': 'common:current_user_edit', 'famfam': 'vcard_edit'}

link_about = {'text': _('About'), 'view': 'common:about_view', 'famfam': 'information'}
link_license = {'text': _('License'), 'view': 'common:license_view', 'famfam': 'script'}

link_current_user_locale_profile_details = {'text': _('Locale profile'), 'view': 'common:current_user_locale_profile_details', 'famfam': 'world'}
link_current_user_locale_profile_edit = {'text': _('Edit locale profile'), 'view': 'common:current_user_locale_profile_edit', 'famfam': 'world_edit'}

link_logout = {'text': _('Logout'), 'view': 'common:logout_view', 'famfam': 'door_out'}



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from acls.api import class_permissions
from common.utils import encapsulate
from documents.models import Document
from navigation.api import (
    register_links, register_model_list_columns, register_top_menu
)
from navigation.links import link_spacer
from rest_api.classes import APIEndPoint

from .links import (
    multiple_documents_selection_tag_remove,
    single_document_multiple_tag_remove, tag_acl_list, tag_attach, tag_create,
    tag_delete, tag_document_list, tag_edit, tag_list, tag_multiple_attach,
    tag_multiple_delete, tag_tagged_item_list
)
from .models import Tag
from .permissions import (
    PERMISSION_TAG_ATTACH, PERMISSION_TAG_DELETE, PERMISSION_TAG_EDIT,
    PERMISSION_TAG_REMOVE, PERMISSION_TAG_VIEW
)
from .widgets import get_tags_inline_widget_simple, single_tag_widget

class_permissions(Document, [
    PERMISSION_TAG_ATTACH, PERMISSION_TAG_REMOVE,
])
class_permissions(Tag, [
    PERMISSION_TAG_DELETE, PERMISSION_TAG_EDIT, PERMISSION_TAG_VIEW,
])

APIEndPoint('tags')

register_model_list_columns(Tag, [
    {
        'name': _('Preview'),
        'attribute': encapsulate(lambda x: single_tag_widget(x))
    },
    {
        'name': _('Tagged items'),
        'attribute': encapsulate(lambda x: x.documents.count())
    }
])

register_model_list_columns(Document, [
    {
        'name': _('Tags'), 'attribute':
        encapsulate(lambda x: get_tags_inline_widget_simple(x))
    },
])

register_top_menu('tags', link={'text': _('Tags'), 'view': 'tags:tag_list', 'famfam': 'tag_blue'})

register_links(Tag, [tag_tagged_item_list, tag_edit, tag_acl_list, tag_delete])
register_links([Tag], [tag_multiple_delete], menu_name='multi_item_links')
register_links([Tag, 'tags:tag_list', 'tags:tag_create'], [tag_list, tag_create], menu_name='secondary_menu')

register_links(Document, [tag_document_list], menu_name='form_header')
register_links(['tags:document_tags', 'tags:tag_remove', 'tags:tag_multiple_remove', 'tags:tag_attach'], [tag_attach], menu_name='sidebar')
register_links(['tags:document_tags'], [single_document_multiple_tag_remove], menu_name='multi_item_links')
register_links([Document], [tag_multiple_attach, multiple_documents_selection_tag_remove, link_spacer], menu_name='multi_item_links')



from __future__ import unicode_literals

from pytz import common_timezones

from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _

from solo.models import SingletonModel

from .managers import AnonymousUserSingletonManager
from .runtime import shared_storage_backend

SHARED_UPLOADED_FILE_PATH = 'shared_uploads'


def upload_to(instance, filename):
    instance.filename = filename
    return '/'.join([SHARED_UPLOADED_FILE_PATH, filename])


class AnonymousUserSingleton(SingletonModel):
    objects = AnonymousUserSingletonManager()

    def __unicode__(self):
        return ugettext('Anonymous user')

    class Meta:
        verbose_name = verbose_name_plural = _('Anonymous user')


class AutoAdminSingleton(SingletonModel):
    account = models.ForeignKey(User, null=True, blank=True, related_name='auto_admin_account', verbose_name=_('Account'))
    password = models.CharField(null=True, blank=True, verbose_name=_('Password'), max_length=128)
    password_hash = models.CharField(null=True, blank=True, verbose_name=_('Password hash'), max_length=128)

    class Meta:
        verbose_name = verbose_name_plural = _('Auto admin properties')


class SharedUploadedFile(models.Model):
    file = models.FileField(upload_to=upload_to, storage=shared_storage_backend, verbose_name=_('File'))
    filename = models.CharField(max_length=255, verbose_name=_('Filename'))
    datatime = models.DateTimeField(auto_now_add=True, verbose_name=_('Date time'))

    class Meta:
        verbose_name = _('Shared uploaded file')
        verbose_name_plural = _('Shared uploaded files')

    def __unicode__(self):
        return self.filename

    def delete(self, *args, **kwargs):
        self.file.storage.delete(self.file.path)
        return super(SharedUploadedFile, self).delete(*args, **kwargs)


class UserLocaleProfile(models.Model):
    user = models.OneToOneField(User, related_name='locale_profile', verbose_name=_('User'))

    timezone = models.CharField(choices=zip(common_timezones, common_timezones), max_length=48, verbose_name=_('Timezone'))
    language = models.CharField(choices=settings.LANGUAGES, max_length=8, verbose_name=_('Language'))

    def __unicode__(self):
        return unicode(self.user)

    class Meta:
        verbose_name = _('User locale profile')
        verbose_name_plural = _('User locale profiles')



from __future__ import unicode_literals

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _

from common.utils import encapsulate
from documents.models import Document
from navigation.api import register_links, register_model_list_columns
from project_setup.api import register_setup

from .models import (
    Workflow, WorkflowInstance, WorkflowInstanceLogEntry, WorkflowState,
    WorkflowTransition
)
from .links import (
    link_document_workflow_instance_list, link_setup_workflow_document_types,
    link_setup_workflow_create, link_setup_workflow_delete,
    link_setup_workflow_edit, link_setup_workflow_list,
    link_setup_workflow_states, link_setup_workflow_state_create,
    link_setup_workflow_state_delete, link_setup_workflow_state_edit,
    link_setup_workflow_transitions, link_setup_workflow_transition_create,
    link_setup_workflow_transition_delete, link_setup_workflow_transition_edit,
    link_workflow_instance_detail, link_workflow_instance_transition
)


@receiver(post_save, dispatch_uid='launch_workflow', sender=Document)
def launch_workflow(sender, instance, created, **kwargs):
    if created:
        Workflow.objects.launch_for(instance)


register_setup(link_setup_workflow_list)

register_model_list_columns(Workflow, [
    {
        'name': _('Initial state'),
        'attribute': encapsulate(lambda workflow: workflow.get_initial_state() or _('None'))
    },
])

register_model_list_columns(WorkflowState, [
    {
        'name': _('Is initial state?'),
        'attribute': 'initial'
    },
])

register_model_list_columns(WorkflowInstance, [
    {
        'name': _('Current state'),
        'attribute': 'get_current_state'
    },
    {
        'name': _('User'),
        'attribute': encapsulate(lambda workflow: getattr(workflow.get_last_log_entry(), 'user', _('None')))
    },
    {
        'name': _('Last transition'),
        'attribute': 'get_last_transition'
    },
    {
        'name': _('Date and time'),
        'attribute': encapsulate(lambda workflow: getattr(workflow.get_last_log_entry(), 'datetime', _('None')))
    },
])

register_model_list_columns(WorkflowTransition, [
    {
        'name': _('Origin state'),
        'attribute': 'origin_state'
    },
    {
        'name': _('Destination state'),
        'attribute': 'destination_state'
    },
])

register_model_list_columns(WorkflowInstanceLogEntry, [
    {
        'name': _('Date and time'),
        'attribute': 'datetime'
    },
    {
        'name': _('User'),
        'attribute': 'user'
    },
    {
        'name': _('Transition'),
        'attribute': 'transition'
    },
    {
        'name': _('Comment'),
        'attribute': 'comment'
    },
])

register_links([Document], [link_document_workflow_instance_list], menu_name='form_header')
register_links([WorkflowInstance], [link_workflow_instance_detail, link_workflow_instance_transition])
register_links([Workflow, 'document_states:setup_workflow_create', 'document_states:setup_workflow_list'], [link_setup_workflow_list, link_setup_workflow_create], menu_name='secondary_menu')
register_links([Workflow], [link_setup_workflow_states, link_setup_workflow_transitions, link_setup_workflow_document_types, link_setup_workflow_edit, link_setup_workflow_delete])
register_links([Workflow], [link_setup_workflow_state_create, link_setup_workflow_transition_create], menu_name='sidebar')
register_links([WorkflowState], [link_setup_workflow_state_edit, link_setup_workflow_state_delete])
register_links([WorkflowTransition], [link_setup_workflow_transition_edit, link_setup_workflow_transition_delete])



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions.models import PermissionNamespace, Permission

user_management_namespace = PermissionNamespace('user_management', _('User management'))

PERMISSION_USER_CREATE = Permission.objects.register(user_management_namespace, 'user_create', _('Create new users'))
PERMISSION_USER_EDIT = Permission.objects.register(user_management_namespace, 'user_edit', _('Edit existing users'))
PERMISSION_USER_VIEW = Permission.objects.register(user_management_namespace, 'user_view', _('View existing users'))
PERMISSION_USER_DELETE = Permission.objects.register(user_management_namespace, 'user_delete', _('Delete existing users'))

PERMISSION_GROUP_CREATE = Permission.objects.register(user_management_namespace, 'group_create', _('Create new groups'))
PERMISSION_GROUP_EDIT = Permission.objects.register(user_management_namespace, 'group_edit', _('Edit existing groups'))
PERMISSION_GROUP_VIEW = Permission.objects.register(user_management_namespace, 'group_view', _('View existing groups'))
PERMISSION_GROUP_DELETE = Permission.objects.register(user_management_namespace, 'group_delete', _('Delete existing groups'))



from __future__ import absolute_import, unicode_literals

from json import dumps, loads

from django.contrib import messages
from django.contrib.auth.models import User
from django.contrib.auth.views import login, password_change
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, render_to_response
from django.template import RequestContext
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _
from django.views.generic import FormView
from django.views.generic.edit import CreateView, DeleteView, UpdateView
from django.views.generic.list import ListView

from .forms import (
    ChoiceForm, EmailAuthenticationForm, LicenseForm, LocaleProfileForm,
    LocaleProfileForm_view, UserForm, UserForm_view
)
from .mixins import (
    ExtraContextMixin, ObjectListPermissionFilterMixin,
    ObjectPermissionCheckMixin, RedirectionMixin, ViewPermissionCheckMixin
)
from .settings import LOGIN_METHOD


def multi_object_action_view(request):
    """
    Proxy view called first when using a multi object action, which
    then redirects to the appropiate specialized view
    """

    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse('main:home'))))

    action = request.GET.get('action', None)
    id_list = ','.join([key[3:] for key in request.GET.keys() if key.startswith('pk_')])
    items_property_list = [loads(key[11:]) for key in request.GET.keys() if key.startswith('properties_')]

    if not action:
        messages.error(request, _('No action selected.'))
        return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse('main:home')))

    if not id_list and not items_property_list:
        messages.error(request, _('Must select at least one item.'))
        return HttpResponseRedirect(request.META.get('HTTP_REFERER', reverse('main:home')))

    # Separate redirects to keep backwards compatibility with older
    # functions that don't expect a properties_list parameter
    if items_property_list:
        return HttpResponseRedirect('%s?%s' % (
            action,
            urlencode({'items_property_list': dumps(items_property_list), 'next': next}))
        )
    else:
        return HttpResponseRedirect('%s?%s' % (
            action,
            urlencode({'id_list': id_list, 'next': next}))
        )


def get_obj_from_content_type_string(string):
    model, pk = string.split(',')
    ct = ContentType.objects.get(model=model)
    return ct.get_object_for_this_type(pk=pk)


def assign_remove(request, left_list, right_list, add_method, remove_method, left_list_title=None, right_list_title=None, decode_content_type=False, extra_context=None, grouped=False):
    left_list_name = 'left_list'
    right_list_name = 'right_list'

    if request.method == 'POST':
        if '%s-submit' % left_list_name in request.POST.keys():
            unselected_list = ChoiceForm(request.POST,
                                         prefix=left_list_name,
                                         choices=left_list())
            if unselected_list.is_valid():
                for selection in unselected_list.cleaned_data['selection']:
                    if grouped:
                        flat_list = []
                        for group in left_list():
                            flat_list.extend(group[1])
                    else:
                        flat_list = left_list()

                    label = dict(flat_list)[selection]
                    if decode_content_type:
                        selection_obj = get_obj_from_content_type_string(selection)
                    else:
                        selection_obj = selection
                    try:
                        add_method(selection_obj)
                    except:
                        if settings.DEBUG:
                            raise
                        else:
                            messages.error(request, _('Unable to remove %(selection)s.') % {
                                'selection': label, 'right_list_title': right_list_title})

        elif '%s-submit' % right_list_name in request.POST.keys():
            selected_list = ChoiceForm(request.POST,
                                       prefix=right_list_name,
                                       choices=right_list())
            if selected_list.is_valid():
                for selection in selected_list.cleaned_data['selection']:
                    if grouped:
                        flat_list = []
                        for group in right_list():
                            flat_list.extend(group[1])
                    else:
                        flat_list = right_list()

                    label = dict(flat_list)[selection]
                    if decode_content_type:
                        selection = get_obj_from_content_type_string(selection)
                    try:
                        remove_method(selection)
                    except:
                        if settings.DEBUG:
                            raise
                        else:
                            messages.error(request, _('Unable to add %(selection)s.') % {
                                'selection': label, 'right_list_title': right_list_title})
    unselected_list = ChoiceForm(prefix=left_list_name, choices=left_list())
    selected_list = ChoiceForm(prefix=right_list_name, choices=right_list())

    context = {
        'subtemplates_list': [
            {
                'name': 'main/generic_form_subtemplate.html',
                'grid': 12,
                'context': {
                    'form': unselected_list,
                    'title': left_list_title or ' ',
                    'submit_label': _('Add'),
                    'submit_icon_famfam': 'add'
                }
            },
            {
                'name': 'main/generic_form_subtemplate.html',
                'grid': 12,
                'grid_clear': True,
                'context': {
                    'form': selected_list,
                    'title': right_list_title or ' ',
                    'submit_label': _('Remove'),
                    'submit_icon_famfam': 'delete'
                }
            },

        ],
    }
    if extra_context:
        context.update(extra_context)

    return render_to_response('main/generic_form.html', context,
                              context_instance=RequestContext(request))


def current_user_details(request):
    """
    Display the current user's details
    """
    form = UserForm_view(instance=request.user)

    return render_to_response(
        'main/generic_form.html', {
            'form': form,
            'title': _('Current user details'),
            'read_only': True,
        },
        context_instance=RequestContext(request))


def current_user_locale_profile_details(request):
    """
    Display the current user's locale profile details
    """
    form = LocaleProfileForm_view(instance=request.user.locale_profile)

    return render_to_response(
        'main/generic_form.html', {
            'form': form,
            'title': _('Current user locale profile details'),
            'read_only': True,
        },
        context_instance=RequestContext(request))


def current_user_edit(request):
    """
    Allow an user to edit his own details
    """

    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse('common:current_user_details'))))

    if request.method == 'POST':
        form = UserForm(instance=request.user, data=request.POST)
        if form.is_valid():
            if User.objects.filter(email=form.cleaned_data['email']).exclude(pk=request.user.pk).count():
                messages.error(request, _('E-mail conflict, another user has that same email.'))
            else:
                form.save()
                messages.success(request, _('Current user\'s details updated.'))
                return HttpResponseRedirect(next)
    else:
        form = UserForm(instance=request.user)

    return render_to_response(
        'main/generic_form.html', {
            'form': form,
            'next': next,
            'title': _('Edit current user details'),
        },
        context_instance=RequestContext(request))


def current_user_locale_profile_edit(request):
    """
    Allow an user to edit his own locale profile
    """

    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse('common:current_user_locale_profile_details'))))

    if request.method == 'POST':
        form = LocaleProfileForm(instance=request.user.locale_profile, data=request.POST)
        if form.is_valid():
            form.save()

            if hasattr(request, 'session'):
                request.session['django_language'] = form.cleaned_data['language']
                request.session['django_timezone'] = form.cleaned_data['timezone']
            else:
                request.set_cookie(settings.LANGUAGE_COOKIE_NAME, form.cleaned_data['language'])

            messages.success(request, _('Current user\'s locale profile details updated.'))
            return HttpResponseRedirect(next)
    else:
        form = LocaleProfileForm(instance=request.user.locale_profile)

    return render_to_response(
        'main/generic_form.html', {
            'form': form,
            'next': next,
            'title': _('Edit current user locale profile details'),
        },
        context_instance=RequestContext(request))


def login_view(request):
    """
    Control how the use is to be authenticated, options are 'email' and
    'username'
    """
    kwargs = {'template_name': 'main/login.html'}

    if LOGIN_METHOD == 'email':
        kwargs['authentication_form'] = EmailAuthenticationForm

    if not request.user.is_authenticated():
        context = {'web_theme_view_type': 'plain'}
    else:
        context = {}

    return login(request, extra_context=context, **kwargs)


def license_view(request):
    """
    Display the included LICENSE file from the about menu
    """
    form = LicenseForm()
    return render_to_response(
        'main/generic_detail.html', {
            'form': form,
            'title': _('License'),
        },
        context_instance=RequestContext(request))


def password_change_view(request):
    """
    Password change wrapper for better control
    """
    context = {'title': _('Current user password change')}

    return password_change(
        request,
        extra_context=context,
        template_name='main/password_change_form.html',
        post_change_redirect=reverse('common:password_change_done'),
    )


def password_change_done(request):
    """
    View called when the new user password has been accepted
    """

    messages.success(request, _('Your password has been successfully changed.'))
    return redirect('common:current_user_details')


class SingleObjectEditView(ViewPermissionCheckMixin, ObjectPermissionCheckMixin, ExtraContextMixin, RedirectionMixin, UpdateView):
    template_name = 'main/generic_form.html'

    def form_invalid(self, form):
        result = super(SingleObjectEditView, self).form_invalid(form)

        try:
            messages.error(self.request, _('Error saving %s details.') % self.extra_context['object_name'])
        except KeyError:
            messages.error(self.request, _('Error saving details.'))

        return result

    def form_valid(self, form):
        result = super(SingleObjectEditView, self).form_valid(form)

        try:
            messages.success(self.request, _('%s details saved successfully.') % self.extra_context['object_name'].capitalize())
        except KeyError:
            messages.success(self.request, _('Details saved successfully.'))

        return result


class SingleObjectCreateView(ViewPermissionCheckMixin, ExtraContextMixin, RedirectionMixin, CreateView):
    template_name = 'main/generic_form.html'

    def form_invalid(self, form):
        result = super(SingleObjectCreateView, self).form_invalid(form)

        try:
            messages.error(self.request, _('Error creating new %s.') % self.extra_context['object_name'])
        except KeyError:
            messages.error(self.request, _('Error creating object.'))

        return result

    def form_valid(self, form):
        result = super(SingleObjectCreateView, self).form_valid(form)
        try:
            messages.success(self.request, _('%s created successfully.') % self.extra_context['object_name'].capitalize())
        except KeyError:
            messages.success(self.request, _('New object created successfully.'))

        return result


class SingleObjectDeleteView(ViewPermissionCheckMixin, ObjectPermissionCheckMixin, ExtraContextMixin, RedirectionMixin, DeleteView):
    template_name = 'main/generic_confirm.html'

    def get_context_data(self, **kwargs):
        context = super(SingleObjectDeleteView, self).get_context_data(**kwargs)
        context.update({'delete_view': True})
        return context

    def delete(self, request, *args, **kwargs):
        try:
            result = super(SingleObjectDeleteView, self).delete(request, *args, **kwargs)
        except Exception as exception:
            try:
                messages.error(self.request, _('Error deleting %s.') % self.extra_context['object_name'])
            except KeyError:
                messages.error(self.request, _('Error deleting object.'))

            raise exception
        else:
            try:
                messages.success(self.request, _('%s deleted successfully.') % self.extra_context['object_name'].capitalize())
            except KeyError:
                messages.success(self.request, _('Object deleted successfully.'))

            return result


class SingleObjectListView(ViewPermissionCheckMixin, ObjectListPermissionFilterMixin, ExtraContextMixin, RedirectionMixin, ListView):
    template_name = 'main/generic_list.html'


class MultiFormView(FormView):
    prefixes = {}

    prefix = None

    def get_form_kwargs(self, form_name):
        kwargs = {}
        kwargs.update({'initial': self.get_initial(form_name)})
        kwargs.update({'prefix': self.get_prefix(form_name)})

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })

        return kwargs

    def _create_form(self, form_name, klass):
        form_kwargs = self.get_form_kwargs(form_name)
        form_create_method = 'create_%s_form' % form_name
        if hasattr(self, form_create_method):
            form = getattr(self, form_create_method)(**form_kwargs)
        else:
            form = klass(**form_kwargs)
        return form

    def get_forms(self, form_classes):
        return dict([(key, self._create_form(key, klass)) for key, klass in form_classes.items()])

    def get_initial(self, form_name):
        initial_method = 'get_%s_initial' % form_name
        if hasattr(self, initial_method):
            return getattr(self, initial_method)()
        else:
            return self.initial.copy()

    def get_prefix(self, form_name):
        return self.prefixes.get(form_name, self.prefix)

    def get(self, request, *args, **kwargs):
        form_classes = self.get_form_classes()
        forms = self.get_forms(form_classes)
        return self.render_to_response(self.get_context_data(forms=forms))

    def forms_valid(self, forms):
        for form_name, form in forms.items():
            form_valid_method = '%s_form_valid' % form_name

            if hasattr(self, form_valid_method):
                return getattr(self, form_valid_method)(form)

        self.all_forms_valid(forms)

        return HttpResponseRedirect(self.get_success_url())

    def forms_invalid(self, forms):
        return self.render_to_response(self.get_context_data(forms=forms))

    def post(self, request, *args, **kwargs):
        form_classes = self.get_form_classes()
        forms = self.get_forms(form_classes)

        if all([form.is_valid() for form in forms.values()]):
            return self.forms_valid(forms)
        else:
            return self.forms_invalid(forms)



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from .links import link_spacer


class MultiItemForm(forms.Form):
    def __init__(self, *args, **kwargs):
        actions = kwargs.pop('actions', [])
        super(MultiItemForm, self).__init__(*args, **kwargs)
        choices = []
        group = []

        for action in actions:
            if not action[0]:
                if group:
                    choices.append((link_spacer['text'], group))
                group = []
            else:
                group.append(action)

        if choices:
            self.fields['action'].choices = choices
        else:
            self.fields['action'].choices = group

    action = forms.ChoiceField(label=_('Actions'), required=False)



from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, reverse_lazy
from django.db.utils import IntegrityError
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django.views.generic import FormView

from acls.models import AccessEntry
from common.utils import generate_choices_w_labels
from common.views import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectEditView,
    SingleObjectListView, assign_remove
)
from documents.models import Document
from permissions.models import Permission

from .forms import (
    WorkflowForm,
    WorkflowInstanceDetailForm, WorkflowInstanceTransitionForm,
    WorkflowStateForm, WorkflowTransitionForm
)
from .models import Workflow, WorkflowInstance, WorkflowState, WorkflowTransition
from .permissions import (
    PERMISSION_WORKFLOW_CREATE, PERMISSION_WORKFLOW_DELETE,
    PERMISSION_WORKFLOW_EDIT, PERMISSION_WORKFLOW_VIEW,
    PERMISSION_DOCUMENT_WORKFLOW_VIEW, PERMISSION_DOCUMENT_WORKFLOW_TRANSITION
)


class DocumentWorkflowInstanceListView(SingleObjectListView):
    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_WORKFLOW_VIEW])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_DOCUMENT_WORKFLOW_VIEW, request.user, self.get_document())

        return super(DocumentWorkflowInstanceListView, self).dispatch(request, *args, **kwargs)

    def get_document(self):
        return get_object_or_404(Document, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_document().workflows.all()

    def get_context_data(self, **kwargs):
        context = super(DocumentWorkflowInstanceListView, self).get_context_data(**kwargs)
        context.update(
            {
                'hide_link': True,
                'object': self.get_document(),
                'title': _('Workflows for document: %s') % self.get_document(),
                'list_object_variable_name': 'workflow_instance',
            }
        )

        return context


class WorkflowInstanceDetailView(SingleObjectListView):
    template_name = 'main/generic_multi_subtemplates.html'

    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_WORKFLOW_VIEW])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_DOCUMENT_WORKFLOW_VIEW, request.user, self.get_workflow_instance().document)

        return super(WorkflowInstanceDetailView, self).dispatch(request, *args, **kwargs)

    def get_workflow_instance(self):
        return get_object_or_404(WorkflowInstance, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_workflow_instance().log_entries.order_by('-datetime')

    def get_context_data(self, **kwargs):
        form = WorkflowInstanceDetailForm(
            instance=self.get_workflow_instance(), extra_fields=[
                {'label': _('Current state'), 'field': 'get_current_state'},
                {'label': _('Last transition'), 'field': 'get_last_transition'},
            ]
        )

        context = {
            'object': self.get_workflow_instance().document,
            'workflow_instance': self.get_workflow_instance(),
            'navigation_object_list': [
                {'object': 'object'},
                {'object': 'workflow_instance'}
            ],
            'title': _('Detail of workflow: %(workflow)s') % {
                'workflow': self.get_workflow_instance()
            },
            'subtemplates_list': [
                {
                    'name': 'main/generic_detail_subtemplate.html',
                    'context': {
                        'form': form,
                    }
                },
                {
                    'name': 'main/generic_list_subtemplate.html',
                    'context': {
                        'object_list': self.get_queryset(),
                        'title': _('Log entries'),
                        'hide_object': True,
                    }
                }
            ]
        }

        return context


class WorkflowInstanceTransitionView(FormView):
    form_class = WorkflowInstanceTransitionForm
    template_name = 'main/generic_form.html'

    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_WORKFLOW_TRANSITION])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_DOCUMENT_WORKFLOW_TRANSITION, request.user, self.get_workflow_instance().document)

        return super(WorkflowInstanceTransitionView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        transition = self.get_workflow_instance().workflow.transitions.get(pk=form.cleaned_data['transition'])
        self.get_workflow_instance().do_transition(comment=form.cleaned_data['comment'], transition=transition, user=self.request.user)
        return HttpResponseRedirect(self.get_success_url())

    def get_form_kwargs(self):
        kwargs = super(WorkflowInstanceTransitionView, self).get_form_kwargs()
        kwargs['workflow'] = self.get_workflow_instance()
        return kwargs

    def get_workflow_instance(self):
        return get_object_or_404(WorkflowInstance, pk=self.kwargs['pk'])

    def get_context_data(self, **kwargs):
        context = super(WorkflowInstanceTransitionView, self).get_context_data(**kwargs)

        context.update(
            {
                'object': self.get_workflow_instance().document,
                'workflow_instance': self.get_workflow_instance(),
                'navigation_object_list': [
                    {'object': 'object'},
                    {'object': 'workflow_instance'}
                ],
                'title': _('Do transition for workflow: %s') % self.get_workflow_instance(),
                'submit_label': _('Submit'),
            }
        )

        return context

    def get_success_url(self):
        return self.get_workflow_instance().get_absolute_url()


# Setup

class SetupWorkflowListView(SingleObjectListView):
    extra_context = {
        'title': _('Workflows'),
        'hide_link': True,
    }
    model = Workflow
    view_permission = PERMISSION_WORKFLOW_VIEW


class SetupWorkflowCreateView(SingleObjectCreateView):
    form_class = WorkflowForm
    model = Workflow
    view_permission = PERMISSION_WORKFLOW_CREATE
    success_url = reverse_lazy('document_states:setup_workflow_list')


class SetupWorkflowEditView(SingleObjectEditView):
    form_class = WorkflowForm
    model = Workflow
    view_permission = PERMISSION_WORKFLOW_EDIT
    success_url = reverse_lazy('document_states:setup_workflow_list')


class SetupWorkflowDeleteView(SingleObjectDeleteView):
    model = Workflow
    view_permission = PERMISSION_WORKFLOW_DELETE
    success_url = reverse_lazy('document_states:setup_workflow_list')


# States

class SetupWorkflowStateListView(SingleObjectListView):
    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_WORKFLOW_EDIT])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_WORKFLOW_EDIT, request.user, self.get_workflow())

        return super(SetupWorkflowStateListView, self).dispatch(request, *args, **kwargs)

    def get_workflow(self):
        return get_object_or_404(Workflow, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_workflow().states.all()

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowStateListView, self).get_context_data(**kwargs)
        context.update(
            {
                'hide_link': True,
                'object': self.get_workflow(),
                'title': _('States of workflow: %s') % self.get_workflow()
            }
        )

        return context


class SetupWorkflowStateCreateView(SingleObjectCreateView):
    form_class = WorkflowStateForm

    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_WORKFLOW_EDIT])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_WORKFLOW_EDIT, request.user, self.get_workflow())

        return super(SetupWorkflowStateCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowStateCreateView, self).get_context_data(**kwargs)
        context.update(
            {
                'object': self.get_workflow(),
                'title': _('Create states for workflow: %s') % self.get_workflow()
            }
        )
        return context

    def get_workflow(self):
        return get_object_or_404(Workflow, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_workflow().states.all()

    def get_success_url(self):
        return reverse('document_states:setup_workflow_states', args=[self.kwargs['pk']])

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.workflow = self.get_workflow()
        self.object.save()
        return super(SetupWorkflowStateCreateView, self).form_valid(form)


class SetupWorkflowStateDeleteView(SingleObjectDeleteView):
    model = WorkflowState
    view_permission = PERMISSION_WORKFLOW_DELETE

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowStateDeleteView, self).get_context_data(**kwargs)

        context.update(
            {
                'object': self.get_object().workflow,
                'workflow_instance': self.get_object(),
                'navigation_object_list': [
                    {'object': 'object'},
                    {'object': 'workflow_instance'}
                ],
            }
        )

        return context

    def get_success_url(self):
        return reverse('document_states:setup_workflow_States', args=[self.get_object().workflow.pk])


class SetupWorkflowStateEditView(SingleObjectEditView):
    form_class = WorkflowStateForm
    model = WorkflowState
    view_permission = PERMISSION_WORKFLOW_EDIT

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowStateEditView, self).get_context_data(**kwargs)

        context.update(
            {
                'object': self.get_object().workflow,
                'workflow_instance': self.get_object(),
                'navigation_object_list': [
                    {'object': 'object'},
                    {'object': 'workflow_instance'}
                ],
            }
        )

        return context

    def get_success_url(self):
        return reverse('document_states:setup_workflow_states', args=[self.get_object().workflow.pk])


# Transitions


class SetupWorkflowTransitionListView(SingleObjectListView):
    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_WORKFLOW_EDIT])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_WORKFLOW_EDIT, request.user, self.get_workflow())

        return super(SetupWorkflowTransitionListView, self).dispatch(request, *args, **kwargs)

    def get_workflow(self):
        return get_object_or_404(Workflow, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_workflow().transitions.all()

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowTransitionListView, self).get_context_data(**kwargs)
        context.update(
            {
                'hide_link': True,
                'object': self.get_workflow(),
                'title': _('Transitions of workflow: %s') % self.get_workflow()
            }
        )

        return context


class SetupWorkflowTransitionCreateView(SingleObjectCreateView):
    form_class = WorkflowTransitionForm

    def dispatch(self, request, *args, **kwargs):
        try:
            Permission.objects.check_permissions(request.user, [PERMISSION_WORKFLOW_EDIT])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_WORKFLOW_EDIT, request.user, self.get_workflow())

        return super(SetupWorkflowTransitionCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowTransitionCreateView, self).get_context_data(**kwargs)
        context.update(
            {
                'object': self.get_workflow(),
                'title': _('Create transitions for workflow: %s') % self.get_workflow()
            }
        )
        return context

    def get_form_kwargs(self):
        kwargs = super(SetupWorkflowTransitionCreateView, self).get_form_kwargs()
        kwargs['workflow'] = self.get_workflow()
        return kwargs

    def get_workflow(self):
        return get_object_or_404(Workflow, pk=self.kwargs['pk'])

    def get_queryset(self):
        return self.get_workflow().transitions.all()

    def get_success_url(self):
        return reverse('document_states:setup_workflow_transitions', args=[self.kwargs['pk']])

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.workflow = self.get_workflow()
        try:
            self.object.save()
        except IntegrityError:
            messages.error(self.request, _('Unable to save transition; integrity error.'))
            return super(SetupWorkflowTransitionCreateView, self).form_invalid(form)
        else:
            return HttpResponseRedirect(self.get_success_url())


class SetupWorkflowTransitionDeleteView(SingleObjectDeleteView):
    model = WorkflowTransition
    view_permission = PERMISSION_WORKFLOW_DELETE

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowTransitionDeleteView, self).get_context_data(**kwargs)

        context.update(
            {
                'object': self.get_object().workflow,
                'workflow_instance': self.get_object(),
                'navigation_object_list': [
                    {'object': 'object'},
                    {'object': 'workflow_instance'}
                ],
            }
        )

        return context

    def get_success_url(self):
        return reverse('document_states:setup_workflow_transitions', args=[self.get_object().workflow.pk])


class SetupWorkflowTransitionEditView(SingleObjectEditView):
    form_class = WorkflowTransitionForm
    model = WorkflowTransition
    view_permission = PERMISSION_WORKFLOW_EDIT

    def get_context_data(self, **kwargs):
        context = super(SetupWorkflowTransitionEditView, self).get_context_data(**kwargs)

        context.update(
            {
                'object': self.get_object().workflow,
                'workflow_instance': self.get_object(),
                'navigation_object_list': [
                    {'object': 'object'},
                    {'object': 'workflow_instance'}
                ],
            }
        )

        return context

    def get_form_kwargs(self):
        kwargs = super(SetupWorkflowTransitionEditView, self).get_form_kwargs()
        kwargs['workflow'] = self.get_object().workflow
        return kwargs

    def get_success_url(self):
        return reverse('document_states:setup_workflow_transitions', args=[self.get_object().workflow.pk])


def setup_workflow_document_types(request, pk):
    workflow = get_object_or_404(Workflow, pk=pk)

    try:
        Permission.objects.check_permissions(request.user, [PERMISSION_WORKFLOW_EDIT])
    except PermissionDenied:
        AccessEntry.objects.check_access(PERMISSION_WORKFLOW_EDIT, request.user, workflow)

    return assign_remove(
        request,
        left_list=lambda: generate_choices_w_labels(workflow.get_document_types_not_in_workflow(), display_object_type=False),
        right_list=lambda: generate_choices_w_labels(workflow.document_types.all(), display_object_type=False),
        add_method=lambda x: workflow.document_types.add(x),
        remove_method=lambda x: workflow.document_types.remove(x),
        decode_content_type=True,
        extra_context={
            'main_title': _('Document types assigned the workflow: %s') % workflow,
            'object': workflow,
        }
    )



from __future__ import unicode_literals

import logging
import sys
import traceback

from django.conf import settings

from documents.models import DocumentVersion
from lock_manager import Lock, LockError
from mayan.celery import app

from .api import do_document_ocr
from .literals import LOCK_EXPIRE
from .models import DocumentVersionOCRError

logger = logging.getLogger(__name__)


@app.task(ignore_result=True)
def task_do_ocr(document_version_pk):
    lock_id = 'task_do_ocr_doc_version-%d' % document_version_pk
    try:
        logger.debug('trying to acquire lock: %s', lock_id)
        # Acquire lock to avoid doing OCR on the same document version more than
        # once concurrently
        lock = Lock.acquire_lock(lock_id, LOCK_EXPIRE)
        logger.debug('acquired lock: %s', lock_id)
        document_version = None
        try:
            logger.info('Starting document OCR for document version: %d', document_version_pk)
            document_version = DocumentVersion.objects.get(pk=document_version_pk)
            do_document_ocr(document_version)
        except Exception as exception:
            logger.error('OCR error for document version: %d; %s', document_version_pk, exception)
            if document_version:
                entry, created = DocumentVersionOCRError.objects.get_or_create(document_version=document_version)

                if settings.DEBUG:
                    result = []
                    type, value, tb = sys.exc_info()
                    result.append('%s: %s' % (type.__name__, value))
                    result.extend(traceback.format_tb(tb))
                    entry.result = '\n'.join(result)
                else:
                    entry.result = exception

                entry.save()
        else:
            logger.info('OCR for document: %d ended', document_version_pk)
            try:
                entry = DocumentVersionOCRError.objects.get(document_version=document_version)
            except DocumentVersionOCRError.DoesNotExist:
                pass
            else:
                entry.delete()
        finally:
            lock.release()
    except LockError:
        logger.debug('unable to obtain lock: %s' % lock_id)
        pass



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from dynamic_search.classes import SearchModel
from permissions.models import Permission

from .api import diagnostics, tools
from .classes import FrontPageButton


def home(request):
    document_search = SearchModel.get('documents.Document')

    context = {
        'object_navigation_links': FrontPageButton.get_all(),
        'query_string': request.GET,
        'hide_links': True,
        'search_results_limit': 100,
    }

    if request.GET:
        queryset, ids, timedelta = document_search.search(request.GET, request.user)

        # Update the context with the search results
        context.update({
            'object_list': queryset,
            'time_delta': timedelta,
            'title': _('Results'),
        })

    return render_to_response('main/home.html', context, context_instance=RequestContext(request))


def maintenance_menu(request):
    user_tools = {}
    for namespace, values in tools.items():
        user_tools[namespace] = {
            'title': values['title']
        }
        user_tools[namespace].setdefault('links', [])
        for link in values['links']:
            try:
                permissions = link.get('permissions', [])
                Permission.objects.check_permissions(request.user, permissions)
                user_tools[namespace]['links'].append(link)
            except PermissionDenied:
                pass

    return render_to_response('main/tools.html', {
        'blocks': user_tools,
        'title': _('Maintenance menu')
    }, context_instance=RequestContext(request))


def diagnostics_view(request):
    return render_to_response('main/diagnostics.html', {
        'blocks': diagnostics,
        'title': _('Diagnostics')
    }, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'MetadataType.validation'
        db.add_column(u'metadata_metadatatype', 'validation',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=64),
                      keep_default=False)

    def backwards(self, orm):
        # Deleting field 'MetadataType.validation'
        db.delete_column(u'metadata_metadatatype', 'validation')

    models = {
        u'documents.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'documents'", 'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'default': "u'Uninitialized document'", 'max_length': '255', 'db_index': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "u'eng'", 'max_length': '8'}),
            'uuid': ('django.db.models.fields.CharField', [], {'default': "u'8fd13d23-fcd0-438c-a1b3-7421a1d0eed5'", 'max_length': '48'})
        },
        u'documents.documenttype': {
            'Meta': {'ordering': "['name']", 'object_name': 'DocumentType'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'ocr': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'metadata.documentmetadata': {
            'Meta': {'unique_together': "(('document', 'metadata_type'),)", 'object_name': 'DocumentMetadata'},
            'document': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'metadata'", 'to': u"orm['documents.Document']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'metadata_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['metadata.MetadataType']"}),
            'value': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'metadata.documenttypemetadatatype': {
            'Meta': {'unique_together': "(('document_type', 'metadata_type'),)", 'object_name': 'DocumentTypeMetadataType'},
            'document_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'metadata'", 'to': u"orm['documents.DocumentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'metadata_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['metadata.MetadataType']"}),
            'required': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        u'metadata.metadatatype': {
            'Meta': {'ordering': "('title',)", 'object_name': 'MetadataType'},
            'default': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lookup': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '48'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '48'}),
            'validation': ('django.db.models.fields.CharField', [], {'max_length': '64'})
        }
    }

    complete_apps = ['metadata']



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _

from acls.models import AccessEntry
from common.models import SharedUploadedFile
from common.utils import encapsulate
from common.views import MultiFormView
from documents.models import DocumentType, Document
from documents.permissions import (
    PERMISSION_DOCUMENT_CREATE, PERMISSION_DOCUMENT_NEW_VERSION
)
from documents.tasks import task_upload_new_version
from metadata.api import decode_metadata_from_url, metadata_repr_as_list
from permissions.models import Permission

from .forms import (
    NewDocumentForm, NewVersionForm, SourceTransformationForm,
    SourceTransformationForm_create
)
from .literals import (
    SOURCE_CHOICE_STAGING, SOURCE_CHOICE_WEB_FORM, SOURCE_UNCOMPRESS_CHOICE_ASK,
    SOURCE_UNCOMPRESS_CHOICE_Y
)
from .models import (
    InteractiveSource, Source, StagingFolderSource, SourceTransformation,
    WebFormSource
)
from .permissions import (
    PERMISSION_SOURCES_SETUP_CREATE, PERMISSION_SOURCES_SETUP_DELETE,
    PERMISSION_SOURCES_SETUP_EDIT, PERMISSION_SOURCES_SETUP_VIEW
)
from .tasks import task_source_upload_document
from .utils import get_class, get_form_class, get_upload_form_class


def document_create_siblings(request, document_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_CREATE])

    document = get_object_or_404(Document, pk=document_id)
    query_dict = {}
    for pk, metadata in enumerate(document.metadata.all()):
        query_dict['metadata%s_id' % pk] = metadata.metadata_type_id
        query_dict['metadata%s_value' % pk] = metadata.value

    query_dict['document_type_id'] = document.document_type_id

    url = reverse('sources:upload_interactive')
    return HttpResponseRedirect('%s?%s' % (url, urlencode(query_dict)))


def conditional_highlight_factory(obj):
    return lambda context: context['source'].source_type == obj.source_type and context['source'].pk == obj.pk


def get_tab_link_for_source(source, document=None):
    if document:
        view = 'sources:upload_version'
        args = [document.pk, source.pk]
    else:
        view = 'sources:upload_interactive'
        args = [source.pk]

    return {
        'text': source.title,
        'view': view,
        'args': args,
        'keep_query': True,
        'remove_from_query': ['page'],
        'conditional_highlight': conditional_highlight_factory(source),
        'famfam': 'application_form',
    }


def get_active_tab_links(document=None):
    tab_links = []

    web_forms = WebFormSource.objects.filter(enabled=True)
    for web_form in web_forms:
        tab_links.append(get_tab_link_for_source(web_form, document))

    staging_folders = StagingFolderSource.objects.filter(enabled=True)
    for staging_folder in staging_folders:
        tab_links.append(get_tab_link_for_source(staging_folder, document))

    return {
        'tab_links': tab_links,
        SOURCE_CHOICE_WEB_FORM: web_forms,
        SOURCE_CHOICE_STAGING: staging_folders,
    }


class UploadBaseView(MultiFormView):
    template_name = 'main/generic_form.html'
    prefixes = {'source_form': 'source', 'document_form': 'document'}

    def dispatch(self, request, *args, **kwargs):
        if 'source_id' in kwargs:
            self.source = get_object_or_404(Source.objects.filter(enabled=True).select_subclasses(), pk=kwargs['source_id'])
        else:
            self.source = InteractiveSource.objects.filter(enabled=True).select_subclasses().first()

        if InteractiveSource.objects.filter(enabled=True).count() == 0:
            messages.error(request, _('No interactive document sources have been defined or none have been enabled, create one before proceeding.'))
            return HttpResponseRedirect(reverse('sources:setup_source_list'))

        return super(UploadBaseView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(UploadBaseView, self).get_context_data(**kwargs)
        subtemplates_list = []

        context['source'] = self.source

        if isinstance(self.source, StagingFolderSource):
            try:
                staging_filelist = list(self.source.get_files())
            except Exception as exception:
                messages.error(self.request, exception)
                staging_filelist = []
            finally:
                subtemplates_list = [
                    {
                        'name': 'main/generic_multiform_subtemplate.html',
                        'context': {
                            'forms': context['forms'],
                        }
                    },
                    {
                        'name': 'main/generic_list_subtemplate.html',
                        'context': {
                            'title': _('Files in staging path'),
                            'object_list': staging_filelist,
                            'hide_link': True,
                        }
                    },
                ]
        else:
            subtemplates_list.append({
                'name': 'main/generic_multiform_subtemplate.html',
                'context': {
                    'forms': context['forms'],
                    'is_multipart': True
                },
            })

        context.update({
            'subtemplates_list': subtemplates_list,
            'extra_navigation_links': {
                'form_header': {
                    'sources:upload_version': {
                        'links': self.tab_links['tab_links']
                    },
                    'sources:upload_interactive': {
                        'links': self.tab_links['tab_links']
                    }
                }
            },
        })

        return context


class UploadInteractiveView(UploadBaseView):

    def dispatch(self, request, *args, **kwargs):
        self.subtemplates_list = []

        Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_CREATE])

        self.document_type = get_object_or_404(DocumentType, pk=self.request.GET.get('document_type_id', self.request.POST.get('document_type_id')))

        self.tab_links = get_active_tab_links()

        return super(UploadInteractiveView, self).dispatch(request, *args, **kwargs)

    def forms_valid(self, forms):
        if self.source.uncompress == SOURCE_UNCOMPRESS_CHOICE_ASK:
            expand = forms['source_form'].cleaned_data.get('expand')
        else:
            if self.source.uncompress == SOURCE_UNCOMPRESS_CHOICE_Y:
                expand = True
            else:
                expand = False

        uploaded_file = self.source.get_upload_file_object(forms['source_form'].cleaned_data)

        shared_uploaded_file = SharedUploadedFile.objects.create(file=uploaded_file.file)

        label = shared_uploaded_file.filename
        if 'document_type_available_filenames' in forms['document_form'].cleaned_data:
            if forms['document_form'].cleaned_data['document_type_available_filenames']:
                label = forms['document_form'].cleaned_data['document_type_available_filenames'].filename

        if not self.request.user.is_anonymous():
            user_id = self.request.user.pk
        else:
            user_id = None

        try:
            self.source.clean_up_upload_file(uploaded_file)
        except Exception as exception:
            messages.error(self.request, exception)

        task_source_upload_document.apply_async(kwargs=dict(
            description=forms['document_form'].cleaned_data.get('description'),
            document_type_id=self.document_type.pk,
            expand=expand,
            label=label,
            language=forms['document_form'].cleaned_data.get('language'),
            metadata_dict_list=decode_metadata_from_url(self.request.GET),
            shared_uploaded_file_id=shared_uploaded_file.pk,
            source_id=self.source.pk,
            user_id=user_id,
        ), queue='uploads')
        messages.success(self.request, _('New document queued for uploaded and will be available shortly.'))
        return HttpResponseRedirect(self.request.get_full_path())

    def create_source_form_form(self, **kwargs):
        return self.get_form_classes()['source_form'](
            prefix=kwargs['prefix'],
            source=self.source,
            show_expand=(self.source.uncompress == SOURCE_UNCOMPRESS_CHOICE_ASK),
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def create_document_form_form(self, **kwargs):
        return self.get_form_classes()['document_form'](
            prefix=kwargs['prefix'],
            document_type=self.document_type,
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def get_form_classes(self):
        return {'document_form': NewDocumentForm, 'source_form': get_upload_form_class(self.source.source_type)}

    def get_context_data(self, **kwargs):
        context = super(UploadInteractiveView, self).get_context_data(**kwargs)
        context['title'] = _('Upload a local document from source: %s') % self.source.title

        context.update(
            {
                'sidebar_subtemplates_list': [
                    {
                        'name': 'main/generic_subtemplate.html',
                        'context': {
                            'title': _('Current document type'),
                            'paragraphs': [self.document_type if self.document_type else _('None')],
                            'side_bar': True,
                        }
                    },
                    {
                        'name': 'main/generic_subtemplate.html',
                        'context': {
                            'title': _('Current metadata'),
                            'paragraphs': metadata_repr_as_list(decode_metadata_from_url(self.request.GET)),
                            'side_bar': True,
                        }
                    }
                ],
            }
        )
        return context


class UploadInteractiveVersionView(UploadBaseView):
    def dispatch(self, request, *args, **kwargs):

        self.subtemplates_list = []

        self.document = get_object_or_404(Document, pk=kwargs['document_pk'])
        try:
            Permission.objects.check_permissions(self.request.user, [PERMISSION_DOCUMENT_NEW_VERSION])
        except PermissionDenied:
            AccessEntry.objects.check_access(PERMISSION_DOCUMENT_NEW_VERSION, self.request.user, self.document)

        self.tab_links = get_active_tab_links(self.document)

        return super(UploadInteractiveVersionView, self).dispatch(request, *args, **kwargs)

    def forms_valid(self, forms):
        uploaded_file = self.source.get_upload_file_object(forms['source_form'].cleaned_data)

        shared_uploaded_file = SharedUploadedFile.objects.create(file=uploaded_file.file)

        try:
            self.source.clean_up_upload_file(uploaded_file)
        except Exception as exception:
            messages.error(self.request, exception)

        if not self.request.user.is_anonymous():
            user_id = self.request.user.pk
        else:
            user_id = None

        task_upload_new_version.apply_async(kwargs=dict(
            shared_uploaded_file_id=shared_uploaded_file.pk,
            document_id=self.document.pk,
            user_id=user_id,
            comment=forms['document_form'].cleaned_data.get('comment')
        ), queue='uploads')

        messages.success(self.request, _('New document version queued for uploaded and will be available shortly.'))
        return HttpResponseRedirect(reverse('documents:document_version_list', args=[self.document.pk]))

    def create_source_form_form(self, **kwargs):
        return self.get_form_classes()['source_form'](
            prefix=kwargs['prefix'],
            source=self.source,
            show_expand=False,
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def create_document_form_form(self, **kwargs):
        return self.get_form_classes()['document_form'](
            prefix=kwargs['prefix'],
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def get_form_classes(self):
        return {'document_form': NewVersionForm, 'source_form': get_upload_form_class(self.source.source_type)}

    def get_context_data(self, **kwargs):
        context = super(UploadInteractiveVersionView, self).get_context_data(**kwargs)
        context['object'] = self.document
        context['title'] = _('Upload a new version from source: %s') % self.source.title

        return context


def staging_file_delete(request, staging_folder_pk, encoded_filename):
    Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_CREATE, PERMISSION_DOCUMENT_NEW_VERSION])
    staging_folder = get_object_or_404(StagingFolderSource, pk=staging_folder_pk)

    staging_file = staging_folder.get_file(encoded_filename=encoded_filename)
    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        try:
            staging_file.delete()
            messages.success(request, _('Staging file delete successfully.'))
        except Exception as exception:
            messages.error(request, _('Staging file delete error; %s.') % exception)
        return HttpResponseRedirect(next)

    results = get_active_tab_links()

    return render_to_response('main/generic_confirm.html', {
        'source': staging_folder,
        'delete_view': True,
        'object': staging_file,
        'next': next,
        'previous': previous,
        'extra_navigation_links': {'form_header': {'staging_file_delete': {'links': results['tab_links']}}},
    }, context_instance=RequestContext(request))


# Setup views
def setup_source_list(request):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_VIEW])

    context = {
        'object_list': Source.objects.select_subclasses(),
        'title': _('Sources'),
        'hide_link': True,
        'list_object_variable_name': 'source',
        'extra_columns': [
            {
                'name': _('Type'),
                'attribute': encapsulate(lambda x: x.class_fullname())
            },
            {
                'name': _('Enabled'),
                'attribute': encapsulate(lambda x: _('Yes') if x.enabled else _('No'))
            },
        ]
    }

    return render_to_response('main/generic_list.html', context,
                              context_instance=RequestContext(request))


def setup_source_edit(request, source_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_EDIT])

    source = get_object_or_404(Source.objects.select_subclasses(), pk=source_id)
    form_class = get_form_class(source.source_type)

    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        form = form_class(instance=source, data=request.POST)
        if form.is_valid():
            try:
                form.save()
                messages.success(request, _('Source edited successfully'))
                return HttpResponseRedirect(next)
            except Exception as exception:
                messages.error(request, _('Error editing source; %s') % exception)
    else:
        form = form_class(instance=source)

    return render_to_response('main/generic_form.html', {
        'title': _('Edit source: %s') % source,
        'form': form,
        'source': source,
        'navigation_object_name': 'source',
        'next': next,
        'source_type': source.source_type,
    }, context_instance=RequestContext(request))


def setup_source_delete(request, source_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_DELETE])
    source = get_object_or_404(Source.objects.select_subclasses(), pk=source_id)
    redirect_view = reverse('sources:setup_source_list')

    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', redirect_view)))

    if request.method == 'POST':
        try:
            source.delete()
            messages.success(request, _('Source "%s" deleted successfully.') % source)
        except Exception as exception:
            messages.error(request, _('Error deleting source "%(source)s": %(error)s') % {
                'source': source, 'error': exception
            })
        return HttpResponseRedirect(redirect_view)

    context = {
        'title': _('Are you sure you wish to delete the source: %s?') % source,
        'source': source,
        'navigation_object_name': 'source',
        'delete_view': True,
        'previous': previous,
        'source_type': source.source_type,
    }

    return render_to_response('main/generic_confirm.html', context,
                              context_instance=RequestContext(request))


def setup_source_create(request, source_type):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_CREATE])

    cls = get_class(source_type)
    form_class = get_form_class(source_type)

    if request.method == 'POST':
        form = form_class(data=request.POST)
        if form.is_valid():
            try:
                form.save()
                messages.success(request, _('Source created successfully'))
                return HttpResponseRedirect(reverse('sources:setup_source_list'))
            except Exception as exception:
                messages.error(request, _('Error creating source; %s') % exception)
    else:
        form = form_class()

    return render_to_response('main/generic_form.html', {
        'title': _('Create new source of type: %s') % cls.class_fullname(),
        'form': form,
        'source_type': source_type,
        'navigation_object_name': 'source',
    }, context_instance=RequestContext(request))


def setup_source_transformation_list(request, source_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_EDIT])

    source = get_object_or_404(Source.objects.select_subclasses(), pk=source_id)

    context = {
        'object_list': SourceTransformation.transformations.get_for_object(source),
        'title': _('Transformations for: %s') % source.fullname(),
        'source': source,
        'navigation_object_name': 'source',
        'list_object_variable_name': 'transformation',
        'extra_columns': [
            {'name': _('Order'), 'attribute': 'order'},
            {'name': _('Transformation'), 'attribute': encapsulate(lambda x: x.get_transformation_display())},
            {'name': _('Arguments'), 'attribute': 'arguments'}
        ],
        'hide_link': True,
        'hide_object': True,
    }

    return render_to_response('main/generic_list.html', context,
                              context_instance=RequestContext(request))


def setup_source_transformation_edit(request, transformation_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_EDIT])

    source_transformation = get_object_or_404(SourceTransformation, pk=transformation_id)
    redirect_view = reverse('sources:setup_source_transformation_list', args=[source_transformation.content_object.pk])
    next = request.POST.get('next', request.GET.get('next', request.META.get('HTTP_REFERER', redirect_view)))

    if request.method == 'POST':
        form = SourceTransformationForm(instance=source_transformation, data=request.POST)
        if form.is_valid():
            try:
                form.save()
                messages.success(request, _('Source transformation edited successfully'))
                return HttpResponseRedirect(next)
            except Exception as exception:
                messages.error(request, _('Error editing source transformation; %s') % exception)
    else:
        form = SourceTransformationForm(instance=source_transformation)

    return render_to_response('main/generic_form.html', {
        'title': _('Edit transformation: %s') % source_transformation,
        'form': form,
        'source': source_transformation.content_object,
        'transformation': source_transformation,
        'navigation_object_list': [
            {'object': 'source', 'name': _('Source')},
            {'object': 'transformation', 'name': _('Transformation')}
        ],
        'next': next,
    }, context_instance=RequestContext(request))


def setup_source_transformation_delete(request, transformation_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_EDIT])

    source_transformation = get_object_or_404(SourceTransformation, pk=transformation_id)
    redirect_view = reverse('sources:setup_source_transformation_list', args=[source_transformation.content_object.pk])
    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', redirect_view)))

    if request.method == 'POST':
        try:
            source_transformation.delete()
            messages.success(request, _('Source transformation deleted successfully.'))
        except Exception as exception:
            messages.error(request, _('Error deleting source transformation; %(error)s') % {
                'error': exception}
            )
        return HttpResponseRedirect(redirect_view)

    return render_to_response('main/generic_confirm.html', {
        'delete_view': True,
        'transformation': source_transformation,
        'source': source_transformation.content_object,
        'navigation_object_list': [
            {'object': 'source', 'name': _('Source')},
            {'object': 'transformation', 'name': _('Transformation')}
        ],
        'title': _('Are you sure you wish to delete source transformation "%(transformation)s"') % {
            'transformation': source_transformation.get_transformation_display(),
        },
        'previous': previous,
    }, context_instance=RequestContext(request))


def setup_source_transformation_create(request, source_id):
    Permission.objects.check_permissions(request.user, [PERMISSION_SOURCES_SETUP_EDIT])

    source = get_object_or_404(Source.objects.select_subclasses(), pk=source_id)

    redirect_view = reverse('sources:setup_source_transformation_list', args=[source.pk])

    if request.method == 'POST':
        form = SourceTransformationForm_create(request.POST)
        if form.is_valid():
            try:
                source_tranformation = form.save(commit=False)
                source_tranformation.content_object = source
                source_tranformation.save()
                messages.success(request, _('Source transformation created successfully'))
                return HttpResponseRedirect(redirect_view)
            except Exception as exception:
                messages.error(request, _('Error creating source transformation; %s') % exception)
    else:
        form = SourceTransformationForm_create()

    return render_to_response('main/generic_form.html', {
        'form': form,
        'source': source,
        'navigation_object_name': 'source',
        'title': _('Create new transformation for source: %s') % source,
    }, context_instance=RequestContext(request))



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .api_views import (
    APICurrentUserView, APIGroupListView, APIGroupView, APIUserListView,
    APIUserView
)

urlpatterns = patterns(
    'user_management.views',
    url(r'^user/list/$', 'user_list', (), 'user_list'),
    url(r'^user/add/$', 'user_add', (), 'user_add'),
    url(r'^user/(?P<user_id>\d+)/edit/$', 'user_edit', (), 'user_edit'),
    url(r'^user/(?P<user_id>\d+)/delete/$', 'user_delete', (), 'user_delete'),
    url(r'^user/multiple/delete/$', 'user_multiple_delete', (), 'user_multiple_delete'),
    url(r'^user/(?P<user_id>\d+)/set_password/$', 'user_set_password', (), 'user_set_password'),
    url(r'^user/multiple/set_password/$', 'user_multiple_set_password', (), 'user_multiple_set_password'),
    url(r'^user/(?P<user_id>\d+)/groups/$', 'user_groups', (), 'user_groups'),

    url(r'^group/list/$', 'group_list', (), 'group_list'),
    url(r'^group/add/$', 'group_add', (), 'group_add'),
    url(r'^group/(?P<group_id>\d+)/edit/$', 'group_edit', (), 'group_edit'),
    url(r'^group/(?P<group_id>\d+)/delete/$', 'group_delete', (), 'group_delete'),
    url(r'^group/multiple/delete/$', 'group_multiple_delete', (), 'group_multiple_delete'),
    url(r'^group/(?P<group_id>\d+)/members/$', 'group_members', (), 'group_members'),
)

api_urls = patterns(
    '',
    url(r'^groups/$', APIGroupListView.as_view(), name='group-list'),
    url(r'^groups/(?P<pk>[0-9]+)/$', APIGroupView.as_view(), name='group-detail'),
    url(r'^users/$', APIUserListView.as_view(), name='user-list'),
    url(r'^users/(?P<pk>[0-9]+)/$', APIUserView.as_view(), name='user-detail'),
    url(r'^users/current/$', APICurrentUserView.as_view(), name='user-current'),
)



from __future__ import unicode_literals

from django.contrib import admin

from .models import AutoAdminSingleton, SharedUploadedFile

admin.site.register(AutoAdminSingleton)
admin.site.register(SharedUploadedFile)



from __future__ import unicode_literals

from rest_framework import serializers

from .models import RecentSearch


class RecentSearchSerializer(serializers.ModelSerializer):
    class Meta:
        model = RecentSearch
        read_only_fields = ('user', 'query', 'datetime_created', 'hits')


class SearchSerializer(serializers.Serializer):
    results = serializers.CharField()



from __future__ import unicode_literals

from django.contrib.auth.models import Group, User

from rest_framework import generics

from rest_api.filters import MayanObjectPermissionsFilter
from rest_api.permissions import MayanPermission

from .permissions import (
    PERMISSION_GROUP_CREATE, PERMISSION_GROUP_DELETE, PERMISSION_GROUP_EDIT,
    PERMISSION_GROUP_VIEW, PERMISSION_USER_CREATE, PERMISSION_USER_DELETE,
    PERMISSION_USER_EDIT, PERMISSION_USER_VIEW
)
from .serializers import GroupSerializer, UserSerializer


class APIGroupListView(generics.ListCreateAPIView):
    serializer_class = GroupSerializer
    queryset = Group.objects.all()

    permission_classes = (MayanPermission,)
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': [PERMISSION_GROUP_VIEW]}
    mayan_view_permissions = {'POST': [PERMISSION_GROUP_CREATE]}

    def get(self, *args, **kwargs):
        """Returns a list of all the groups."""
        return super(APIGroupListView, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        """Create a new group."""
        return super(APIGroupListView, self).post(*args, **kwargs)


class APIGroupView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = GroupSerializer
    queryset = Group.objects.all()

    permission_classes = (MayanPermission,)
    mayan_object_permissions = {
        'GET': [PERMISSION_GROUP_VIEW],
        'PUT': [PERMISSION_GROUP_EDIT],
        'PATCH': [PERMISSION_GROUP_EDIT],
        'DELETE': [PERMISSION_GROUP_DELETE]
    }

    def delete(self, *args, **kwargs):
        """Delete the selected group."""
        return super(APIGroupView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """Return the details of the selected group."""
        return super(APIGroupView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """Partially edit the selected group."""
        return super(APIGroupView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """Edit the selected group."""
        return super(APIGroupView, self).put(*args, **kwargs)


class APIUserListView(generics.ListCreateAPIView):
    serializer_class = UserSerializer
    queryset = User.objects.all()

    permission_classes = (MayanPermission,)
    filter_backends = (MayanObjectPermissionsFilter,)
    mayan_object_permissions = {'GET': [PERMISSION_USER_VIEW]}
    mayan_view_permissions = {'POST': [PERMISSION_USER_CREATE]}

    def get(self, *args, **kwargs):
        """Returns a list of all the users."""
        return super(APIUserListView, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        """Create a new user."""
        return super(APIUserListView, self).post(*args, **kwargs)


class APIUserView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = UserSerializer
    queryset = User.objects.all()

    permission_classes = (MayanPermission,)
    mayan_object_permissions = {
        'GET': [PERMISSION_USER_VIEW],
        'PUT': [PERMISSION_USER_EDIT],
        'PATCH': [PERMISSION_USER_EDIT],
        'DELETE': [PERMISSION_USER_DELETE]
    }

    def delete(self, *args, **kwargs):
        """Delete the selected user."""
        return super(APIUserView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """Return the details of the selected user."""
        return super(APIUserView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """Partially edit the selected user."""
        return super(APIUserView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """Edit the selected user."""
        return super(APIUserView, self).put(*args, **kwargs)


class APICurrentUserView(generics.RetrieveUpdateDestroyAPIView):
    serializer_class = UserSerializer

    def get_object(self):
        return self.request.user

    def delete(self, *args, **kwargs):
        """Delete the current user."""
        return super(APICurrentUserView, self).delete(*args, **kwargs)

    def get(self, *args, **kwargs):
        """Return the details of the current user."""
        return super(APICurrentUserView, self).get(*args, **kwargs)

    def patch(self, *args, **kwargs):
        """Partially edit the current user."""
        return super(APICurrentUserView, self).patch(*args, **kwargs)

    def put(self, *args, **kwargs):
        """Edit the current user."""
        return super(APICurrentUserView, self).put(*args, **kwargs)



from __future__ import absolute_import, unicode_literals

import pytz

from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404

from rest_framework import generics, status
from rest_framework.response import Response

from acls.models import AccessEntry
from documents.models import Document
from documents.permissions import PERMISSION_DOCUMENT_VIEW
from permissions.models import Permission

from .models import DocumentCheckout
from .permissions import PERMISSION_DOCUMENT_CHECKOUT, PERMISSION_DOCUMENT_CHECKIN, PERMISSION_DOCUMENT_CHECKIN_OVERRIDE
from .serializers import DocumentCheckoutSerializer, NewDocumentCheckoutSerializer


class APICheckedoutDocumentListView(generics.ListCreateAPIView):
    def get_serializer_class(self):
        if self.request.method == 'POST':
            return NewDocumentCheckoutSerializer
        else:
            return DocumentCheckoutSerializer

    def get_queryset(self):
        documents = DocumentCheckout.objects.checked_out_documents()

        try:
            Permission.objects.check_permissions(self.request.user, [PERMISSION_DOCUMENT_VIEW])
        except PermissionDenied:
            filtered_documents = AccessEntry.objects.filter_objects_by_access([PERMISSION_DOCUMENT_VIEW], self.request.user, documents)
        else:
            filtered_documents = documents

        return DocumentCheckout.objects.filter(document__pk__in=filtered_documents.values_list('pk', flat=True))

    def get(self, request, *args, **kwargs):
        """
        Returns a list of all the documents that are currently checked out.
        """
        return super(APICheckedoutDocumentListView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        """
        Checkout a document.
        """
        serializer = self.get_serializer(data=request.DATA, files=request.FILES)

        if serializer.is_valid():
            document = get_object_or_404(Document, pk=serializer.data['document'])
            try:
                Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_CHECKOUT])
            except PermissionDenied:
                AccessEntry.objects.check_access(PERMISSION_DOCUMENT_CHECKOUT, request.user, document)

            timezone = pytz.utc

            try:
                DocumentCheckout.objects.create(
                    document=document,
                    expiration_datetime=timezone.localize(serializer.data['expiration_datetime']),
                    user_object=request.user,
                    block_new_version=serializer.data['block_new_version']
                )
            except Exception as exception:
                return Response(data={'exception': unicode(exception)}, status=status.HTTP_400_BAD_REQUEST)

            return Response(status=status.HTTP_201_CREATED)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class APICheckedoutDocumentView(generics.RetrieveDestroyAPIView):
    serializer_class = DocumentCheckoutSerializer

    def get_queryset(self):
        if self.request.method == 'GET':
            documents = DocumentCheckout.objects.checked_out_documents()

            try:
                Permission.objects.check_permissions(self.request.user, [PERMISSION_DOCUMENT_VIEW])
            except PermissionDenied:
                filtered_documents = AccessEntry.objects.filter_objects_by_access([PERMISSION_DOCUMENT_VIEW], self.request.user, documents)
            else:
                filtered_documents = documents

            return DocumentCheckout.objects.filter(document__pk__in=filtered_documents.values_list('pk', flat=True))
        elif self.request.method == 'DELETE':
            return DocumentCheckout.objects.all()

    def get(self, request, *args, **kwargs):
        """
        Retrieve the details of the selected checked out document entry.
        """

        return super(APICheckedoutDocumentView, self).get(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        """
        Checkin a document.
        """

        document = self.get_object().document

        if document.checkout_info().user_object == request.user:
            try:
                Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_CHECKIN])
            except PermissionDenied:
                AccessEntry.objects.check_access(PERMISSION_DOCUMENT_CHECKIN, request.user, document)
        else:
            try:
                Permission.objects.check_permissions(request.user, [PERMISSION_DOCUMENT_CHECKIN_OVERRIDE])
            except PermissionDenied:
                AccessEntry.objects.check_access(PERMISSION_DOCUMENT_CHECKIN_OVERRIDE, request.user, document)

        return super(APICheckedoutDocumentView, self).delete(request, *args, **kwargs)



from __future__ import unicode_literals

from django.contrib import admin

from .models import MetadataType


class MetadataTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'title', 'default', 'lookup', 'validation')


admin.site.register(MetadataType, MetadataTypeAdmin)



from __future__ import unicode_literals

from rest_framework import generics
from rest_framework.exceptions import ParseError

from rest_api.filters import MayanObjectPermissionsFilter

from .classes import SearchModel
from .models import RecentSearch
from .serializers import RecentSearchSerializer, SearchSerializer


class APIRecentSearchListView(generics.ListAPIView):
    """
    Returns a list of all the recent searches.
    """

    serializer_class = RecentSearchSerializer
    queryset = RecentSearch.objects.all()

    # TODO: Add filter_backend so that users can only see their own entries


class APIRecentSearchView(generics.RetrieveDestroyAPIView):
    """
    Returns the selected recent search details.
    """

    serializer_class = RecentSearchSerializer
    queryset = RecentSearch.objects.all()

    # TODO: Add filter_backend so that users can only see their own entries


class APISearchView(generics.ListAPIView):
    """
    Perform a search operaton
    q -- Term that will be used for the search.
    """

    filter_backends = (MayanObjectPermissionsFilter,)

    # Placeholder serializer to avoid errors with Django REST swagger
    serializer_class = SearchSerializer

    def get_queryset(self):
        document_search = SearchModel.get('documents.Document')
        self.serializer_class = document_search.serializer
        self.mayan_object_permissions = {'GET': [document_search.permission]}

        try:
            queryset, ids, timedelta = document_search.search(self.request.GET, self.request.user)
        except Exception as exception:
            raise ParseError(unicode(exception))

        return queryset



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from navigation.api import register_top_menu
from project_setup.api import register_setup
from project_tools.api import register_tool

from .links import admin_site, maintenance_menu
from .classes import FrontPageButton  # NOQA

register_top_menu('home', link={'text': _('Home'), 'view': 'main:home', 'famfam': 'house'}, position=0)
register_setup(admin_site)
register_tool(maintenance_menu)



from __future__ import unicode_literals

from django.conf import settings

DEFAULT_LOCK_TIMEOUT_VALUE = 30

DEFAULT_LOCK_TIMEOUT = getattr(settings, 'LOCK_MANAGER_DEFAULT_LOCK_TIMEOUT', DEFAULT_LOCK_TIMEOUT_VALUE)
